;+=----------------------------------------------------------------------------
; Name   : mrCursorInfo
; RCS    : $Id: mrcursorinfo__define.pro,v 2.0 2004/11/10 10:42:21 olivert Exp olivert $
; Methods: <b>Constructor</b>
;          mrCursorInfo::init                : constructor, part II
;          <br>
;          <b>General</b>
;          mrCursorInfo::show                : show the info
;          mrCursorInfo::hide                : don't show
;          mrCursorInfo::toggle              : toggle show/no-show
;          <br>
;          <b>Event handling</b>
;          mrCursorInfo::selfregister        : register with a display object
;          mrCursorInfo::unregister          : unregister
;          <br>
;          <b>Private</b>
;          mrCursorInfo::mrDisplayEvtHandler : the event handler and broker for mr*display
;          mrCursorInfo::update              : update text string. Does not redisplay.
;          mrCursorInfo__define              : the constructor
;          mrCursorInfo::cleanup             : the destructor
; Desc   : A plugin object for mrdisplay which displays the current image coordinates
;          and image value next to the cross-hair cursor
; Author : (c) Oliver Thilmann, oliver.thilmann@web.de
; Licns  : This program is published under the GPL. See gpl.txt for details
;=-----------------------------------------------------------------------------


;==============================================================================
;
;                          CONSTRUCTORS AND DESTRUCTORS
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : mrCursorInfo::init
; Syntax: x = obj_new('mrCursorInfo', fontsize=float, color=intarr(3))
; Args  : <b>fontsize</b>
;         fontsize in pixels. Default is 10.
;         <b>color</b>
;         3-element integer array with RGB-values. Default is [0,255,0] (green)
; Desc  : The constructor. Depending on font and fontsize, drawing the
;         graphic window results in 'Floating underflow' errors. Dunno what
;         to do about this.
;=-----------------------------------------------------------------------------
function mrCursorInfo::init, fontsize=fontsize, color=color

	fontsize = (n_elements(fontsize) ne 1) ?       10. : fontsize
	color    = (n_elements(color)    ne 3) ? [0,255,0] : color

	self.onoff     = 0
	self.toggleKey = 0
	self.alignment = 0.

	self.grFont  = obj_new('IDLgrFont', 'courier', size=fontsize)
	self.grModel = obj_new('IDLgrModel', hide=0)
	self.grText  = obj_new('IDLgrText', color=color, alignment=0., font=self.grFont, $
							recompute_dimensions=2, /onglass)
	self.grModel->add, self.grText
	return, 1
end

;+=----------------------------------------------------------------------------
; Name  : mrCursorInfo::cleanup
; Desc  : cleanup at object destruction
;=-----------------------------------------------------------------------------
function mrCursorInfo::cleanup
	if( self.display ne obj_new() ) then begin
		if( obj_valid(self.display) ) then self->unregister
	end
	obj_destroy, self.grModel
	obj_destroy, self.grText
	obj_destroy, self.grFont
end


;==============================================================================
;
;                                PUBLIC METHODS
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mrCursorInfo::show
; Syntax: obj->show
; Desc  : Display info. Will redisplay mrDisplay.
;=-----------------------------------------------------------------------------
pro mrCursorInfo::show
	self.onoff = 1
	self->update
	self.grModel->setProperty, hide=0
	self.display->display
end

;+=----------------------------------------------------------------------------
; Name  : mrCursorInfo::hide
; Syntax: obj->hide
; Desc  : Hide info. Will redisplay mrDisplay.
;=-----------------------------------------------------------------------------
pro mrCursorInfo::hide
	self.onoff = 0
	self.grModel->setProperty, hide=1
	self.display->display
end

;+=----------------------------------------------------------------------------
; Name  : mrCursorInfo::toggle
; Syntax: obj->toggle
; Desc  : Toggles display of cursor info.
;=-----------------------------------------------------------------------------
pro mrCursorInfo::toggle
	self.onoff = 1-self.onoff
	if( self.onoff ) then self->update
	self.grModel->setProperty, hide=1-self.onoff
	self.display->display
end

;==============================================================================
;
;                                  EVENT HANDLING
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : mrCursorInfo::selfregister
; Syntax: obj->selfregister, display, toggleKey='i'
; Args  : <b>display</b>
;         The mrDisplay object with which to register
;         <b>toggleKey</b>
;         A key which will toggle data display on/off. toggleKey may either
;         be a single character or the corresponding ascii (byte) value.
; Desc  : Register with an mrDisplay object
;=-----------------------------------------------------------------------------
pro mrCursorInfo::selfregister, display, toggleKey=toggleKey

	self.toggleKey =  ( n_elements(toggleKey) eq 0 ) ? 0 : byte(toggleKey)
	display->registerEvt, self, ['Motion', 'KeyPress', 'DblClick', 'ImgChange']
	self.display = display

	(display->imgView())->remove, self.grModel
	(display->imgView())->add, self.grModel
	self.grText->setProperty, xcoord_conv=[0.,1./self.display->screenXsize()], $
							  ycoord_conv=[0.,1./self.display->screenySize()]

	return
end

;+=----------------------------------------------------------------------------
; Name  : mrCursorInfo::unregister
; Syntax: obj->unregister
; Desc  : unregisters with a mrdisplay object. It is a good idea to unregister
;         before destroying the mrdisplay or the mrCursorInfo object.
;=-----------------------------------------------------------------------------
pro mrCursorInfo::unregister
	if( self.display eq obj_new() ) then return
	self.display->unregisterObj, self
	(self.display->labelView())->remove, self.grModel
	self.display = obj_new()
end

;+=----------------------------------------------------------------------------
; Name  : mrCursorInfo::mrDisplayEvtHandler
; Syntax: obj->mrDisplayEvtHandler, evt
; Args  : <b>evt</b>
;         Pointer to the mrDisplay event structure
; Desc  : Dispatch the event coming from mrDisplay
;         Double-clicks and image changes might render the last known x and
;         y positions invalid (depends on whether e.g. the mrZoom plugin
;         registered earlier or later than mrCursorInfo - image change events
;         don't contain a valid x/y image position). These events will
;         thus temporarily switch off the display - until the next move event.
;=-----------------------------------------------------------------------------
pro mrCursorInfo::mrdisplayEvtHandler, evt
	case (*evt).type of
		1:  begin												; double click
			self.lastimgx = -1
			self.lastimgy = -1
			self->update
			self.display->requestDisplay
		end
		3:	begin												; motion
			self.lastimgx = (*evt).imgx
			self.lastimgy = (*evt).imgy
			self.lastx    = (*evt).x
			self.lasty    = (*evt).y
			if( self.onoff eq 0 ) then return
			self->update
			self.display->requestDisplay
		end
		4:	begin												; key
			if( (*evt).key ne self.toggleKey ) then return
			self.onoff = 1-self.onoff
			self.grModel->setProperty, hide=1-self.onoff
			self->update
			self.display->requestDisplay
		end
		5: 	begin												; image change
			self.lastimgx = -1
			self.lastimgy = -1
			self->update
		end
		else: return
	end
end


;==============================================================================
;
;                             PRIVATE OBJECT METHODS
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mrCursorInfo::update
; Syntax: obj->update, evt
; Args  : <b>evt</b>
;         Pointer to the mrDisplay event structure
; Desc  : Updates information
;=-----------------------------------------------------------------------------
pro mrCursorInfo::update
	if( self.lastimgx eq -1 or self.lastimgy eq -1 ) then begin
		text = ''
	end else begin
		text = ''
		text = strtrim( self.lastimgx, 2)+'/'+strtrim(self.lastimgy,2)
		img = self.display->img()
		if( ( size(*img))[0] eq 2 ) then begin
			val = fix((*img)[self.lastimgx,self.lastimgy])
			text = text + ';' + strtrim( val, 2 )
		end
		if( self.lastx gt .7*self.display->screenXsize() ) then self.alignment=1
		if( self.lastx lt .3*self.display->screenXsize() ) then self.alignment=0
		self.grText->setProperty, xcoord_conv=[0.,1./self.display->imgXsize()], ycoord_conv=[0.,1./self.display->imgYsize()]
	end
	off = (self.alignment-.5)*8
	self.grText->setProperty, locations=[self.lastimgx-off, self.lastimgy+4], alignment=self.alignment, strings=text
end


;+=----------------------------------------------------------------------------
; Name  : mrCursorInfo__define
; Type  : Private Procedure (constructor)
;=-----------------------------------------------------------------------------
pro mrCursorInfo__define
	obj = { mrCursorInfo, $
			display		:	obj_new()	,$ ; the display object we're associated with
			toggleKey	:	0			,$ ; byte value of the toggle key
			onoff		:   0           ,$ ;
			alignment   :   0.          ,$ ;
			lastImgX	:	0			,$ ; last known x-position (for toggle)
			lastImgY	:	0			,$ ; last known y-position
			lastx		:	0			,$ ; last known absolute x position
			lasty		:	0			,$ ; last known absolute y position
			grModel		:	obj_new() 	,$ ;
			grText		:	obj_new()	,$ ;
			grFont		:	obj_new()	 $ ;
		}
end
