;+=------------------------------------------------------------------
; Name   : mrProgressbar
; RCS    : $Id: mrprogressbar__define.pro,v 2.1 2004/08/19 09:23:58 olivert Exp olivert $
; Methods: <b>Constructor</b>
;          mrProgressbar::init         : constructor
;          <br>
;          <b>Public</b>
;          mrProgressbar::update       : set value between 0. and 1.
;          mrProgressbar::reset        : reset bar to 0.
;          mrProgressbar::cancel       : check for status of cancel button
;          <br>
;          <b>Private</b>
;          mrProgressbar__define       : constructor part I
;          mrProgressbar::cleanup      : destructor
; Desc:
;          An object that implements a progress bar widget. Based on
;          Robert Mallozzi's widget_progress.
; Code:
;
; >
; >   progress = obj_new('mrProgressbar', cancel_label = 'Interrupt')
; >
; >    for i = 0, 20 do begin
; >
; >        fraction = i / 20.0
; >
; >        progress->update, fraction
; >        IF (progress->cancel ()) then $
; >           goto, DONE
; >
; >        ; Replace the WAIT command with the work you want performed
; >        ;
; >        wait, 2
; >
; >    end
; >
; >    DONE:
; >    obj_destroy, progress
;
;
; Author:
;     derived from widget_progress, written April 1999 by Robert Mallozzi
;     (I'm sad to learn that Robert Mallozzi died in Nov 2000)
;     Nov.2002 made cancel_button optional
;     Dec.2002 added reset method
;     Dec.2002 added base option, so progressbars may be added to an existing window
;     Jan.2002 full docu, removed 'map' option and all associated stuff,
;              minor cosmetic adjustments
; Licns : This program is published under the GPL. See gpl.txt for details
;=-------------------------------------------------------------------



;+=----------------------------------------------------------------------------
; Name  : mrProgressbar::init
; Syntax: pb = obj_new('mrProgressbar', parent, xpos=int, ypos=int, title=string, /frame, /remaining, $
;         &nbsp;             cancel_label=label, /cancel_button, /floating, /modal, group_leader=group_leader)
; Args  : <b>parent</b>
;         If supplied, the progressbar will use parent as parent base, otherwise it will
;         create a separate top-level widget as parent.
;         <b>xpos, ypos</b>
;         For top-level widgets, x and y position of the window - if not provided,
;         the progressbar will display in the center of the screen
;         <b>xsize</b>
;         x size of the bar in pixels, default is 300
;         <b>title</b>
;         For top level windows, a string holding the title of the window
;         <b>frame</b>
;         If set, draw a frame around the base_window
;         <b>group_leader</b>
;         Sets the group leader for this widget
;         <b>floating</b>
;         Makes this widget a floating widget - group_leader must be provided in this case.
;         <b>modal</b>
;         Makes the progressbar a modal widget - group_leader must be provided
;         Modal widgets will not stop calculation tasks, they just prevent the user from
;         doing anything else in the meantime
;         <b>remaining</b>
;         If set, estimate remaining time. Possible values are '0' = no estimation (default),
;         '1' for integrated estimation and '2' for adaptive estimation
;         <b>cancel_label</b>
;         Label for the cancel_button - default is 'Cancel'
;         <b>cancel_button</b>
;         Flag - if set, a cancel button is displayed
; Desc:   initialize a mrProgressbar object and display it on screen
;         If the progressbar is created with option /remaining, an estimated
;         remaining time, calculated from starting time and the current time point
;         will be calculated. If set to 2, the remaining time will be calculated
;         from the last 5 calls.
;         Bugs: fgcolor and bgcolor is ignored!
;=----------------------------------------------------------------------------
;         <b>fgcolor</b> not functional
;         3-element integer array denoting the foreground color, default is green
;         <b>bgcolor</b>
;         3-element integer array denoting the background color, default is black
function mrProgressbar::init, parent, title=title, xsize=xsize, xpos=xpos, ypos=ypos, $
    			group_leader=group_leader, cancel_label=cancel_label, bgcolor=bgcolor, fgcolor=fgcolor, $
    			cancel_button=cancel_button, frame=frame, remaining=remaining, floating=floating, modal=modal

	i18n = mrI18n()

	color_init


	baseid		  = (n_elements( parent )        eq 0) ? 0L                       : parent
	title         = (n_elements( title )         eq 0) ? i18n->get('pb_progress') : title
	xsize         = (n_elements( xsize )         eq 0) ? 300                      : xsize
    cancel_label  = (n_elements( cancel_label )  eq 0) ? i18n->get('pb_cancel')   : cancel_label
    cancel_button = (n_elements( cancel_button ) eq 0) ? 0                        : cancel_button
    bgcolor       = (n_elements( bgcolor )       eq 0) ? !p.background            : bgcolor
    fgcolor       = (n_elements( fgcolor )       eq 0) ? !p.color                 : fgcolor
    remaining     = (n_elements( remaining )     eq 0) ? 0                        : remaining

    floating      = keyword_set( floating )
    modal         = keyword_set( modal )
	if( floating and n_elements(group_leader) ne 1 ) then message, 'Missing group_leader keyword'

	if( baseid ne 0L ) then begin
		self.topid = widget_base( baseid, xsize=xsize, /column, /base_align_center, frame=frame )
		if( title ne '' ) then x = widget_label( self.topid, value=string(title) )
	end else begin
		if( n_elements(group_leader) eq 1 ) then begin
			self.topid = widget_base( /column, floating=floating, modal=modal, title=title, group_leader=group_leader, $
									xpad=5, ypad=5, xsize=xsize, /base_align_center )
	    end else begin
	    	self.topid = widget_base( /column, title=title, xpad=5, ypad=5, xsize=xsize, ysize=ysize, /base_align_center )
	    end
	end

	drawID = widget_draw( self.topid, xsize=xsize-10, ysize=8  )
    self.labelID  = widget_label (self.topID, VALUE = ' 0% '+i18n->get(pb_complete), xsize=xsize, /align_center)
    self.cancel_button = cancel_button
    if( cancel_button) then self.cancelID = widget_button (self.topID, VALUE = cancel_label)

	self.flgRemain = remaining
    self.fgColor = color_getidx('lime')
    self.bgColor = color_getidx('white')


    	; Realize, if separate window

    if( baseid eq 0L ) then widget_control, self.topID, /realize

	    ; Fill the progress bar with background color

    widget_control, drawID, get_value = win
    self.winID = win
    wset, self.winID
    polyfill, [0, 1, 1, 0], [0, 0, 1, 1], /normal, color=bgColor

		; position window in the middle of the screen, unless otherwise given

	if( baseid eq 0L ) then begin
		geom   = widget_info( self.topid, /geometry )
		screen = get_screen_size()
		if( n_elements(xpos) ne 0 ) then xp = xpos else	xp = screen[0]/2-geom.scr_xsize/2-geom.margin
		if( n_elements(ypos) ne 0 ) then yp = ypos else yp = screen[1]/2-geom.scr_ysize/2-geom.margin
 		widget_control, self.topid, xoffset=xp, yoffset=yp
 	end

	return, 1
end

;+=----------------------------------------------------------------------------
; Name: mrProgressbar::cleanup
; Desc: destroys the progressbar window
;=----------------------------------------------------------------------------
pro mrProgressbar::cleanup
	if( widget_info( self.topID, /valid) ) then     $
		widget_control, self.topID, /destroy
end


;+=----------------------------------------------------------------------------
; Name  : mrProgressbar::update
; Syntax: obj->update, fraction
; Args  : <b>fraction</b>
;         A floating value between 0. and 1.
; Desc  : Update the progress widget - updating to a smaller value has no
;         visible effect! In that case use mrProgressbar::reset before updating.
;=----------------------------------------------------------------------------
pro mrProgressbar::update, fraction

	i18n = mrI18n()
	txtTimeRemaining = i18n->get('pb_timeRemaining')
	txtComplete      = i18n->get('pb_complete')
	txtSeconds       = i18n->get('pb_seconds')

		; Clamp range to [0.0, 1.0]

    fraction = max( [0.0, fraction] )
    fraction = min( [1.0, fraction] )

		; calculate remainder

	remtxt = ''
	if( self.flgRemain gt 0 and fraction lt 1. ) then begin

		switch self.flgRemain of
		1: begin
			if(  fraction eq 0. ) then begin        ; interpreted as reset
				self.remain[0,0] = systime(1)
				timediff         = 0.
			end else begin
				timediff         = systime(1) - self.remain[0,0]
				progress         = fraction
			end
		end
		2: begin
			nextID   = (self.remainID+1) mod 5
			time     = systime(1)
			timediff = time - self.remain[self.remainID,0];
			if( timediff gt 1. ) then begin             ; we don't save timestamps faster than 1 per second
				self.remain[nextID,0] = time
				self.remain[nextID,1] = fraction
				progress = self.remain[nextID,1] - self.remain[self.remainID,1]
				self.remainID = nextID
			end else begin
				prevID = self.remainID-1
				if( prevID lt 0 ) then prevID = 4
				progress = self.remain[self.remainID,1]-self.remain[prevID,1]
				timediff = self.remain[self.remainID,0]-self.remain[prevID,0]
			end
		end
		end

		if( timediff lt .001 or timediff gt 500. ) then begin
			remtxt = txtTimeRemaining + ' --'
    	end else begin
    		if( progress lt 0. ) then begin
    			remtxt = txtTimeRemaining + ' ups!'
    		end else begin
    			remtime = fix( (1.0-fraction)/progress*timediff )
    			remmin  = fix( remtime/60 )
    			remsec  = remtime - remmin*60
    			if( remmin gt 5 ) then begin
    				remtxt = string( txtTimeRemaining, remmin+1,format='(a,i4," min")' )
    			end else if( remmin gt 0 ) then begin
    				remtxt = string( txtTimeRemaining, remmin, remsec, format='(a,i4," min ",i2," s")' )
    			end else begin
    				remtxt = string( txtTimeRemaining, remsec, txtSeconds, format='(a, " ",i2," ", a)' )
    			end
    		end
	  	end

    end
	pertxt = string( fix(fraction*100), txtComplete, format='(i3,"% ",a)' )

    if( fraction eq 1. ) then begin
      	widget_control, self.labelID, set_value = '100% '+txtComplete
      	self.remain[*,0] = 0.
    end else begin
    	widget_control, self.labelID, set_value = pertxt + '  ' + remtxt
    end

		; Draw updated bar

	wset, self.winID
    polyfill, [0, fraction, fraction, 0], [0, 0, 1, 1], /normal, color = self.fgColor


end

;+=----------------------------------------------------------------------------
; Name  : mrProgressbar::reset
; Syntax: obj->reset()
; Desc  : Resets the mrProgressbar display to 0%
;=----------------------------------------------------------------------------
pro mrProgressbar::reset
	i18n = mrI18n()
	wset, self.winID
	polyfill, [0,1,1,0], [0,0,1,1], /normal, color=0
	widget_control, self.labelID, set_value = ' 0% '+i18n->get('pb_complete')
	self.remain[*,1] = 0.
	self.remain[*,0] = systime(1)
	self.remainID = 0
end

;+=----------------------------------------------------------------------------
; Name  : mrProgressbar::cancel
; Syntax: res = obj->cancel()
; Ret   : '1', if cancelbutton has been pressed, else '0'
; Desc  : Check, whether cancel button has been pressed.
;=----------------------------------------------------------------------------
function mrProgressbar::cancel

	; no cancel button? -> result is always 0

	if( self.cancel_button eq 0 ) then return, 0

    quit = widget_event( self.cancelID, /nowait, /save )
    if( quit.id eq self.cancelID ) then abort = 1 else abort = 0

    return, abort

end


;+=----------------------------------------------------------------------------
; Name: mrProgressbar__define
; Desc: Implementation of a mrProgressbar object
;=----------------------------------------------------------------------------
pro mrProgressbar__define

    obj = { MRPROGRESSBAR, $
        topID    		: 0L			, $
		labelID  		: 0L			, $
		cancelID 		: 0L			, $
		cancel_button	: 0L			, $
        winID    		: 0L			, $
        flgRemain       : 0             , $  ; calculate remaining time
        remain          : dblarr(5,2)   , $  ; array, time, percentage
        remainID        : 0             , $  ; index of the next entry
        bgColor  		: !P.BACKGROUND	, $
        fgColor  		: !P.COLOR 		  $
    }


end
