;+=----------------------------------------------------------------------------
; Name  : mrWidget_fileselect
; RCS   : $Id: mrwidget_fileselect.pro,v 2.0 2004/11/10 11:21:08 olivert Exp olivert $
; Syntax: widget_id = mrwidget_fileselect( parent, scr_xsize=int, scr_ysize=int,
;         .           uvalue=value, filter=string[array], /frame, /no_cancel)
; Ret   : The widget id of the created compound widget or 0L in case of errors.
; Args  : <b>parent</b>
;         Widget id of the parent
;         <b>filter</b>
;         A string or array of strings holding filter settings for the corresponding
;         drop-down list. One entry may contain several filters, separated by whitespace,
;         e.g. ['*', '*.jpg *.tiff', '*.dcm *dicom']
;         <b>scr_xsize, scr_ysize</b>
;         x and y size in pixels on screen
;         <b>frame</b>
;         If set, draw a frame around the widget
;         <b>no_cancel</b>
;         If set, no cancel button is displayed
; Desc  : A compound widget realizing a file-selector box, similar to IDL's cw_filesel,
;         but nicer layout. Only runs on Windows.
;         mrWidget_fileselect events have the following event structure:
;         { MRWIDGET_FILESELECT_EVT, ID:0L, TOP:0L, HANDLER:0L, TYPE:0, PATH:'', FILE:'' }
;
;         Where type is one of:
;         0: the user pressed the 'cancel' button
;         1: the user either single clicked a file or used the cursor keys to select
;            the next file. PATH and FILE are set accordingly
;         2: the user double clicked a file or pressed the open button. Pressing the open
;            button with no file selected will not cause an event.
;
; Author: (c) Oliver Thilmann, oliver.thilmann@web.de
; Licns : This program is published under the GPL. See gpl.txt for details
;=-----------------------------------------------------------------------------
function mrWidget_fileselect, parent, uvalue=uvalue, frame=frame, uname=uname, scr_xsize=xsize, $
							  scr_ysize=ysize, filter=filter, no_cancel=no_cancel

		;
		; process arguments and set defaults
		;

	on_error, 2
	frame      = keyword_set( frame )
	no_cancel  = keyword_set( no_cancel )

	uname     = ( n_elements(uname     ) eq 0 ) ?  '' : uname
	uvalue    = ( n_elements(uvalue    ) eq 0 ) ?   0 : uvalue
	xsize     = ( n_elements(xsize     ) eq 0 ) ? 200 : xsize
	ysize     = ( n_elements(ysize     ) eq 0 ) ? 200 : ysize
	filter    = ( n_elements(filter    ) eq 0 ) ? '*' : filter

	if( n_elements(parent) eq 0 ) then return, 0L

		;
		; set up the widget
		;

	baseid = widget_base( parent, /column, uvalue=uvalue, $
				event_func     = 'mrwidget_fileselect_evt', $ ; widget_event will call this func
				pro_set_value  = 'mrwidget_fileselect_set', $ ; called @ widget_control, set_value
				func_get_value = 'mrwidget_fileselect_get', $ ; called @ widget_control, get_value
				/base_align_center, uname=uname, frame=frame, scr_xsize=xsize, xsize=xsize, $
				scr_ysize=ysize, ysize=ysize )

		; Drive: dropdown_list

	b1 = widget_base( baseid, /row, scr_xsize=xsize-10 )
	x  = widget_label( b1, value='Drive: ', scr_xsize=70 )
	x  = widget_droplist( b1, value=get_drive_list(), scr_xsize=xsize-115 )
	idDrivesel = x

		; button: up1lvl, widget_text containing current directory

	b2 = widget_base( baseid, /row, scr_xsize=xsize-10 )
	x  = widget_text( b2, value='', scr_xsize=xsize-42, /align_right )
	idDir = x
	x  = widget_button( b2, value='up1lvl.bmp', /bitmap, /align_right )
	idUpdir = x

		; widget_text with n lines and vertical scroll bar (if necessary)

	b3 = widget_base( baseid, /column, scr_xsize=xsize-10 )
	x  = widget_list( b3, scr_ysize=ysize-140 )
	idFilelist = x

		; filter

	b4 = widget_base( baseid, /row, scr_xsize=xsize-10 )
	x  = widget_label( b4, value='Filename:', scr_xsize=70 )
	x  = widget_text( b4, value='', scr_xsize=xsize-143 )
	idFilename = x
	x  = widget_button( b4, value='Open', /frame, scr_xsize=50 )
	idOpen = x

		; filename, + 'open' button

	b5 = widget_base( baseid, /row, scr_xsize=xsize-10 )
	x  = widget_label( b5, value='Files of type:', scr_xsize=70 )
	x  = widget_droplist( b5, scr_xsize=xsize-143, value=filter )
	idFiletype = x
	if( not no_cancel ) then begin
		x  = widget_button( b5, value='Cancel', scr_xsize=50 )
		idCancel = x
	end else begin
		idCancel = 0L
	end

		;
		; define the event structure

	x = { mrwidget_fileselect_evt, id:0L, top:0L, handler:0L, type:0, path:'', file:'' }

		;
		; create widget structure and store it
		;

	state = { baseid:baseid, idUpdir:idUpdir, idOpen:idOpen, idCancel:idCancel, idDriveSel:idDriveSel, $
			  idDir:idDir, idFilelist:idFilelist, idFilename: idFilename, idFileType:idFiletype, list:ptr_new(), $
			  currDir:'', currDrive:'', value:'', filterlist:filter, currfilter:filter[0] }

	widget_control, b1, set_uvalue=state
	widget_control, baseid, notify_realize='mrwidget_fileselect_realize'
	if( widget_info(baseid, /visible) eq 1 ) then mrwidget_numfield_realize, baseid
	return, baseid
end


;+=----------------------------------------------------------------------------
; Name  : mrwidget_fileselect_realize
; Desc  : Fill filelist and initialize
;=-----------------------------------------------------------------------------
pro mrwidget_fileselect_realize, id
	child = widget_info( id, /child )
	widget_control, child, get_uvalue=state, /no_copy

	widget_control, state.idDrivesel, get_value=drives
	x = where( drives eq 'C:\' )
	if( x ne -1 ) then 	widget_control, state.idDrivesel, set_droplist_select=x

	ptr_free, state.list
	state.list = mrwidget_fileselect_getFileList( 'C:\', state.currfilter )
	widget_control, state.idFilelist, set_value=*state.list
	widget_control, state.idUpdir, sensitive=0
	state.currDrive = 'C:\'
	state.currDir   = ''

	widget_control, child, set_uvalue=state, /no_copy
end


;+=----------------------------------------------------------------------------
; Name  : mrwidget_fileselect_evt
; Desc  : Returning '0' will swallow the event, otherwise we return a
;         fileselect event.
;=-----------------------------------------------------------------------------
function mrwidget_fileselect_evt, evt

	error_status = 0
	catch, error_status
	if( error_status ne 0 ) then begin
		catch, /cancel
		widget_control, child, set_uvalue=state, /no_copy
		; we catch the error, but do not hand it on
		print, '% MRWIDGET_FILESELECT_EVT: ', + !ERROR_STATE.MSG
		return, 0
	end

	parent = evt.handler
	child  = widget_info( parent, /child )
	widget_control, child, get_uvalue=state, /no_copy


		;
		; Event handling starts here
		;

	case evt.id of

			; Up one directory

		state.idUpdir: begin
			if( state.currDir eq '' ) then return,0  ; should not happen
			x = strpos( state.currdir, '\', 1, /reverse_offset, /reverse_search )
			if( x eq -1 ) then begin
				widget_control, state.idUpdir, sensitive=0
				state.currDir = ''
			end else begin
				state.currDir = strmid( state.currDir, 0, x+1 )
			end
			widget_control, state.idDir, set_value=state.currDrive+state.currDir
			ptr_free, state.list
			print, state.currDir
			state.list = mrwidget_fileselect_getFileList( state.currDrive+state.currDir, state.currfilter )
			widget_control, state.idFilelist, set_value=*state.list
			widget_control, state.idFilename, set_value=''
			ret = 0
		end

			; Cancel button

		state.idCancel:	begin
			widget_control, state.idFilename, set_value=''
			ret = { mrwidget_fileselect_evt, state.baseid, evt.top, evt.handler, 0, '', '' }
		end

			; Selection of a (possibly different) drive

		state.idDriveSel: begin
			widget_control, state.idDriveSel, get_value=list
			drivelistchange=0
			nlist = get_drive_list() ; catch drive changes
			if( n_elements(list) ne n_elements(nlist) ) then begin
				widget_control, state.idDriveSel, set_value=nlist
				drivelistchange=1
			end else begin
				for i = 0, n_elements(list)-1 do begin
					if( list[i] ne nlist[i] ) then begin
						widget_control, state.idDriveSel, set_value=nlist
						drivelistchange=1
						break
					end
				end
			end
			if( drivelistchange ) then begin
				state.currDrive = 'C:\'						; not too clean, but what the heck
				state.currDir   = ''
				x = where( nlist eq 'C:\' )
				widget_control, state.idDrivesel, set_droplist_select=x
			end else begin
				if( state.currDrive eq list[evt.index] ) then break;  do nothing, if not changed
				state.currDrive = list[evt.index]
				state.currDir   = ''
			end
			widget_control, state.idDir, set_value=state.currDrive
			ptr_free, state.list
			state.list = mrwidget_fileselect_getFileList( state.currDrive, state.currfilter )
			widget_control, state.idFileList, set_value=*state.list
			widget_control, state.idFilename, set_value=''
			widget_control, state.idUpdir, sensitive=0
			ret = 0
		end

			; Open button

		state.idOpen: begin
			id = widget_info( state.idFilelist, /list_select )
			if( id eq -1 ) then begin
				ret = 0
				break
			end
			file = (*state.list)[id]
			ret = { mrwidget_fileselect_evt, state.baseid, evt.top, evt.handler, 2, state.currDrive+state.currDir, file }
		end

			; Selection of an entry from the filelist

		state.idFilelist: begin
			case evt.clicks of
				1: begin
					id = widget_info( state.idFilelist, /list_select )
					if( id eq -1 ) then begin ; should not happen
						ret = 0
						break
					end
					file = (*state.list)[id]
					if( strpos(file,'\') ne -1 ) then begin  ; directory selected
						widget_control, state.idFilename, set_value=''
						ret = 0
					end else begin
						widget_control, state.idFilename, set_value=file
						ret = { mrwidget_fileselect_evt, state.baseid, evt.top, evt.handler, 1, state.currDrive+state.currDir, file }
					end
				end
				2: begin
					file = (*state.list)[evt.index]
					if( strpos( file, '\') ne -1 ) then begin			; directory selected
						state.currDir=state.currDir+file
						widget_control, state.idDir, set_value=state.currDrive+state.currDir
						ptr_free, state.list
						state.list = mrwidget_fileselect_getFileList( state.currDrive+state.currDir, state.currfilter )
						widget_control, state.idFilelist, set_value=*state.list
						widget_control, state.idFilename, set_value=''
						widget_control, state.idUpdir, /sensitive
						ret = 0
					end else begin										; file selected
						widget_control, state.idFilename, set_value=file
						ret = { mrwidget_fileselect_evt, state.baseid, evt.top, state.baseid, 2, state.currDrive+state.currDir, file }
					end
				end
				else: begin
					ret = 0
				end
			end
		end

			; Filename is read-only as of now

		state.idFilename: begin
			ret = 0
		end

			; Selection of a different filter setting

		state.idFiletype: begin
			state.currfilter = state.filterlist[ evt.index ]
			ptr_free, state.list
			state.list = mrwidget_fileselect_getFileList( state.currDrive+state.currDir, state.currfilter )
			widget_control, state.idFilelist, set_value=*state.list
			ret = 0
		end

		else: begin
			message, 'unknown event in mrwidget_fileselect', evt.id
			; this will break the widget because state is not reset
		end
	end

		; save state back in base widget 'b1'

	widget_control, child, set_uvalue=state, /no_copy
    return, ret
end


;+=----------------------------------------------------------------------------
; Name  : mrwidget_fileselect_set
; Desc  : Called from widget_control, this_id, set_value=value
;=-----------------------------------------------------------------------------
pro mrwidget_fileselect_set, baseid, value
	b1 = widget_info( baseid, /child )
	widget_control, b1, get_uvalue=state
end


;+=----------------------------------------------------------------------------
; Name  : mrwidget_fileselect_get
; Desc  : Called via widget_control, this_id, get_value=value
;         If a file is selected, the full path to the file is returned, 
;         otherwise only the currently selected path is returned to the user.
;=-----------------------------------------------------------------------------
function mrwidget_fileselect_get, baseid
	b1 = widget_info( baseid, /child )
	widget_control, b1, get_uvalue=state
	id = widget_info( state.idFilelist, /list_select )
	if( id eq -1 ) then begin
		file = ''
	end else begin
		if( id ge n_elements( *state.list ) ) then file = '' else file = (*state.list)[id]
	end
	return, state.currDrive+state.currDir+file
end



;==============================================================================
;
;                               I N T E R N A L
;
;==============================================================================
function mrwidget_fileselect_getFilelist, path, filter

	cd, path

		; get all files in path that are directories or fulfill filter criteria

	if( filter eq 'All files' ) then begin
		allfiles = file_search( '*', /mark_directory )
	end else begin
		allfiles = ''
		dirs = file_search( '*', /test_directory, /mark_directory )
		if( dirs[0] ne '' ) then allfiles = [allfiles, dirs]
		filters = strsplit( filter, ' ', /extract)
		for i = 0, n_elements(filters)-1 do begin
			addfiles = file_search( filters, /mark_directory )
			if( n_elements(addfiles) gt 0  ) then allfiles = [allfiles,addfiles]
		end
		allfiles = allfiles[1:*]
		allfiles = allfiles[sort(allfiles)]
		allfiles = allfiles[uniq(allfiles)]
	end

		; remove possible '..'

	x = where( allfiles eq '..', complement=ids )
	if( x[0] ne -1 ) then allfiles = allfiles[ids]

		; sort

	dirids = where( strpos(allfiles,'\') ne -1, complement=fileids, ncomplement=nfiles )
	files  = ''
	if( dirids[0] ne -1 ) then begin
		dirs = allfiles[dirids]
		dirs = dirs[sort(strupcase(dirs))]
	end
	if( nfiles gt 0 ) then begin
		files = allfiles[fileids]
		files = files[sort(strupcase(files))]
	end
	if( dirs[0] ne '' ) then begin
		if( files[0] ne '' ) then all=[dirs,files] else all=dirs
	end else begin
		if( files[0] ne '' ) then all=files else all=''
	end

	return, ptr_new(all, /no_copy )
end

