;+=----------------------------------------------------------------------------
; Name    : init_gui
; RCS     : $Id: init_gui.pro,v 1.1 2004/11/11 11:13:45 olivert Exp olivert $
; Desc    : Initializes the GUI, i.e.
;           - create the top level widget_base
;           - load menu.xml from disk and create the menu hierarchy while
;             parsing the file
; Author  : (c) 2004 Oliver Thilmann
;=------------------------------------------------------------------------------
function init_gui, g

    topid  = widget_base( mbar=w_menu, column=1, title='Lupe', $
                             tlb_frame_attr=0, xoffset=0, yoffset=0, $
                             xsize=1016, ysize=695, /align_center )

		; parse files and add menus while parsing

	xmlparser = obj_new('menu_xml', w_menu, (*g).i18n, (*g).command, (*g).option )
	xmlparser->parsefile, 'menu.xml'

		; If something went wrong during parsing, errMsg contains a
		; meaningful error message which can be printed to the user.
		; Failure in creating the menu is fatal.

	if( xmlparser->errMsg() ne '' ) then begin
		x = dialog_message( xmlparser->errMsg(), /error )
		return, 0
	end

	return, topid
end


;==============================================================================
;
;                                  XML READER
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : menu_xml::init
; Desc  : The parser reads the xml file and stores the read values in the
;         prefs structure
;         There's no DTD for this file, so we do some 'manual' error checks
;=-----------------------------------------------------------------------------
function menu_xml::init, baseid, i18n, command, option
	self.menuid[0] = baseid
	self.level     = 0
	self.i18n      = i18n
	self.command   = command
	self.option    = option
	self.errmsg    = ''
	self.ce        = -1                         ; ce stands for 'current element'
	return, self->IDLffXMLSAX::init()
end

function menu_xml::errMsg  ; if ne '', something went wrong
	return, self.errmsg
end

;+=----------------------------------------------------------------------------
; Name  : menu_xml::startElement
; Desc  :
;=-----------------------------------------------------------------------------
pro menu_xml::startElement, URI, local, strName, attr, value
	separator = 0
	title     = ''
	case local of
		'menu'    : begin
			for i = 0, n_elements(attr)-1 do begin
				case attr[i] of
					'title'     : title     = value[i]
					'separator' : separator = value[i]
					else: begin
						self.errmsg = 'Illegal menu tag in menu.xml'
						self->stopParsing
					end
				end
			end
			if( title eq '' ) then begin
				self.errmsg = 'Missing title attribute for menu in menu.xml'
				self->stopParsing
			end
			if( self.level eq 4 ) then begin
				self.errmsg = 'Menus nested too deeply in menu.xml (max 4 levels)'
				self->stopParsing
			end
			self.menuid[self.level+1] = widget_button( self.menuid[self.level], value=self.i18n->get(title), uvalue=title, separator=separator, /menu )
			self.level++
			self.ce = 1
		end
		'entry' : begin
			self.ce = 2
			if( n_elements(attr) gt 0 ) then self.sep = fix(value[0]) else self.sep = 0
		end
		'label' : self.ce = 3
		'module': begin
			if( self.label eq '' ) then begin
				self.errmsg = '<module> tag without preceding <label> tag'
				self->stopParsing
			end
			if( n_elements(attr) eq 1 ) then self.option->set, self.label, value[0]
			self.ce = 4
		end
		else	: self.ce = -1  ; unknown element
	end
end

;+=----------------------------------------------------------------------------
; Name  : menu_xml::endElement
; Desc  :
;=-----------------------------------------------------------------------------
pro menu_xml::endElement, URI, local, strName
	case local of
		'menu'  : begin
			self.level--
			self.ce    = -1
		end
		'entry' : begin
			self.ce    = -1
			self.label = ''
		end
		'label' : self.ce = -1
		'module': self.ce = -1
		else    : self.ce = -1 ; jaja, the others are placeholders
	end
end

;+=----------------------------------------------------------------------------
; Name  : menu_xml::characters
; Desc  :
;=-----------------------------------------------------------------------------
pro menu_xml::characters, data
	case self.ce of
	   -1:
		1:
		2:
		3: begin
			x = widget_button( self.menuid[self.level], value=self.i18n->get(data), uvalue=data, separator=self.sep )
			self.label = data
		end
		4: self.command->set, self.label, data
 	end
end

;+=----------------------------------------------------------------------------
; Name  : menu_xml__define
; Desc  : create separate object - dunno if multiple inheritance works under
;         IDL, but that's cleaner anyway (actually multiple inheritance *does*
;         work)
;=-----------------------------------------------------------------------------
pro menu_xml__define
	obj = { MENU_XML, $
			inherits IDLffXMLSAX		,$
			i18n:			 obj_new()  ,$ ; the internationalization hash
			command:         obj_new()  ,$ ; the command hash
			option:          obj_new()  ,$ ; the option hash
			errmsg:                 ''  ,$ ; error message (if any)
			ce:                      0	,$ ; the current element
			menuid:			 lonarr(5)  ,$ ; up to 3 menu levels - ids of the button widgets
			level:                   0  ,$ ; the current level
			label:                  ''  ,$ ; the current label
			sep:                     0   $ ; add a separator before the current element?
		}
end