;+=----------------------------------------------------------------------------
; Name    : lupe_evthandler
; RCS     : $Id: lupe_evthandler.pro,v 1.1 2004/11/11 11:16:18 olivert Exp olivert $
; Desc    : The menu event handler which takes care of mapping and unmapping
;           of modules and possible exclusiveness of a module.
;           Module types are
;
;           'noui'
;           for modules that don't need a UI to run or handle their user
;           interaction exclusively during readyToExecute (e.g. mod_exit)
;
;           'top'
;           for modules that open their own top level window.
;
;           'embedded'
;           modules that map themselves into the main window are 'embedded'
;
;           'noui' and 'top' modules may declare themselves as exclusive. In
;           The case of 'noui' modules, this will cause all other modules to
;           unmap before execution of the 'noui' module.
;           As for 'top' modules, execution of an 'exclusive' top module will
;           unmap all other currently mapped top-level modules.
;           'embedded' modules are always exclusive, as a possibly currently
;           active embedded module needs to be unmapped first.
;           As of now, exclusiveness is only used for mod_exit, as it will
;           make all modules to cleanly unmap (and thus cleanup) before
;           program exit.
;
; Author  : (c) 2004 Oliver Thilmann
;=------------------------------------------------------------------------------
pro lupe_evthandler, evt

		; following a suggestion by David Fanning in comp.lang.idl-pvwave David Fanning
		; each widget which may cause an event has a structure containing the object, 'o',
		; and the object's method, 'm', to call upon the event stored in its uvalue.

	widget_control, evt.id,  get_uvalue=menu_label
	widget_control, evt.top, get_uvalue=g

		; menu_label  : the unique label of the chosen menu entry
		; module_name : the module class, associated with the menu entry
		; module      : all modules are 'singletons' - module_name references the instance

	module_name = (*g).command->get( menu_label )
	module      = (*g).modules->get( module_name )
	option      = (*g).option->get( menu_label )

	if( module eq obj_new() ) then begin
		if( n_elements(module_name) eq 0 ) then module_name = 'Unknown'
		x = dialog_message(['Could not find module.', 'Menu label : ' + menu_label,'Module name: ' + module_name] )
		return
	end

		; we take the chance to update the topmod list - the user might have closed a
		; module via the kill button, and although lupemod catches kills, let's assure
		; that the list is ok
		; lupemods register with xmanager under their name, so we check, if the name
		; is still known to xmanager, and remove the module from the list, if not.

	type = module->type()
	topmods = (*g).top_modules->get( /all, count=count )
	for i = 0, count-1 do begin
		name = topmods[i]->name()
		if( xregistered(name) eq 0 ) then (*g).top_modules->remove, topmods[i]
	end

		; check whether the selected module is ready to run - return immediately if not
		; The argument is the optional 'option' attribute from the <module> tag in menu.xml
		; That way, a module can perform different tasks depending on the menu entry

	res = module->readyToExecute( option )
	if( res eq 0 ) then return

		; ok, the module is ready to run. If it is exclusive, close all other mods

	if( module->exclusive() eq 1 ) then begin

			; If the exclusive module is not embedded, unmap the currently embedded
			; module (unless, of course, it is the logo) - this will implicitly map the logo

		if( ((*g).emb_module ne (*g).modules->get('mod_logo')) and (type eq 2) ) then (*g).emb_module->unmap

			; unmap all mapped top modules

		topmods = (*g).top_modules->get( /all, count=count )
		for i = 0, count-1 do begin
			topmods[i]->unmap
		end
		(*g).top_modules->remove, /all

	end

		; finally execute or map the selected module

	case type of
		0: begin ; noui
			module->execute, option
		end
		1: begin ; top
			(*g).top_modules->add, module
			module->map, option
		end
		2: begin ; embedded
			if( (*g).emb_module ne obj_new() ) then (*g).emb_module->unmap, /nologo
			(*g).emb_module = module
			(*g).emb_module->map, option
		end
	end
end


;+=----------------------------------------------------------------------------
; Name  : lupe_evthandlerInformUnmap
; Desc  : Called by the unmap method from the modules to inform the eventhandler.
;         Remove module from top-level list (if it was in there) and display
;         the logo (if the unmapped module was of embedded type and *not* the
;         logo of course)
;=-----------------------------------------------------------------------------
pro lupe_evthandlerInformUnmap, g, module, nologo=nologo
	nologo = n_elements(nologo) eq 1 ? nologo : 0
	case module->type() of
		0:						; nothing to be done for non-ui modules
		1: (*g).top_modules->remove, module
		2: begin
			(*g).emb_module = obj_new()
			if( nologo eq 1 ) then return
			logomod = (*g).modules->get('mod_logo')
			logomod->map
			(*g).emb_module = logomod
		end
	end
end
