;+=----------------------------------------------------------------------------
; Name    : lupemod
; RCS     : $Id: lupemod__define.pro,v 1.1 2004/11/11 11:17:27 olivert Exp olivert $
; Desc    : This is the template object for a lupe module. Real modules should
;           subclass to this object and overload all methods - the methods
;           implemented here will create an error message upon execution.
;           The menu dispatcher will approximately work the following way:
;           - user selects menu entry, corresponding to a module
;           - call readyToExecute of this module
;             if result is zero, do nothing (it is the module's responsibility
;             to inform the user that it is not ready to run - if it should
;             care to do so)
;           - readyToExecute is ok.
;             If the new module is 'exclusive', unmap all currently mapped
;             modules. If it is of type 'embedded' and there's currently another
;             module in the main window, call the unmap function of the currently
;             embedded module first.
;             The menu dispatcher will also take care to display the logo in
;             main window whenever no embedded window is displayed.
;             Then map the new module and we're done for now
;           - Otherwise 'execute' (non-UI) or 'map' (top-level) the new module.
;             Note that top-level modules may be closed by the user anytime
;             without further notification!
;           - 'mod_exit' would be an example for an exclusive module (to make
;             sure all other modules are closed properly) without UI.
; Author  : (c) 2004 Oliver Thilmann
;=------------------------------------------------------------------------------


;+=----------------------------------------------------------------------------
; Name  : lupemod::init
; Syntax: obj = obj_new('lupemod', type, exclusive, revision )
; Args  : <b>type</b>
;         One of 'noUI', 'top', 'embedded' (case-insensitive)
;         <b>exclusive</b>
;         Either '0' for non-exclusive or '1' for exclusive modules
;         (an exclusive module will cause other modules to unmap before mapping
;         itself)
; Desc  :
;=-----------------------------------------------------------------------------
function lupemod::init, name, type, exclusive, g

	if( n_elements(name     ) eq 0 ) then return, 0
	if( n_elements(exclusive) eq 0 ) then return, 0
	if( n_elements(g        ) eq 0 ) then return, 0
	case strlowcase(type) of
		'noui'    : self.type = 0
		'top'     : self.type = 1
		'embedded': self.type = 2
		else      : return, 0
	end

	self.name      = name
	self.exclusive = exclusive
	self.g         = g
	self.scr_xpos  = -1
	self.scr_ypos  = -1
	self.baseid    = 0L

	return, 1
end

;+=----------------------------------------------------------------------------
; Name  : lupemod::revision
; Syntax: rev = obj->revision()
; Desc  : Returns the current revision of the module as a string
;         In this method we check whether we're (falsely) called from the
;         subclass or directly. All other methods have to be overloaded.
;=-----------------------------------------------------------------------------
function lupemod::revision
	if( obj_class(self) eq 'LUPEMOD' ) then begin
		rev = '$Revision: 1.1 $'
		return, rev
	end else begin
		x = dialog_message(['Overload lupemod::revision','in ' + obj_class(self)])
	end
end

;==============================================================================
;
;                           MODULE HANDLING METHODS
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : lupemod::readyToExecute
; Desc  : The module checks whether it is ready to be mapped to screen. This
;         might involve things like checking whether all necessary data is
;         available to be able to run.
;=-----------------------------------------------------------------------------
function lupemod::readyToExecute, menulabel=label
	x = dialog_message(['Overload lupemod::readyToExecute','in ' + obj_class(self)])
	return, 0
end

;+=----------------------------------------------------------------------------
; Name  : lupemod::map
; Desc  : Map the module to screen
;=-----------------------------------------------------------------------------
pro lupemod::map, menulabel=label
	x = dialog_message(['Overload lupemod::map','in ' + obj_class(self)])
	return
end

;+=----------------------------------------------------------------------------
; Name  : lupemod::unmap
; Desc  : Remove the module from screen. If there's no cleanup to do, the
;         method doesn't need to be implemented by the module. However if so,
;         unmapping shall not be done in the overloaded method as some bookkeeping
;         is done here as well. Hold on to the template.
;         The nologo option is used from lupe_evthandler to avoid mapping of
;         the logo when another module is to be mapped anyway
;=-----------------------------------------------------------------------------
pro lupemod::unmap, killed=killed, nologo=nologo

	nologo = n_elements(nologo) eq 1 ? nologo : 0

	if( self.baseid ne 0L ) then begin
		if( widget_info( self.baseid, /valid_id ) ne 0 ) then begin
			widget_control, self.baseid, /destroy
		end
		self.baseid = 0L
	end

		; The top event handler doesn't know we've been unmapped, so we tell him

	lupe_evthandlerInformUnmap, self.g, self, nologo=nologo
end

;+=----------------------------------------------------------------------------
; Name  : lupemod::execute
; Desc  : Modules of type '0' (no UI) cannot be mapped to screen. Instead their
;         execute function is called to fulfill Map the module on screen
;=-----------------------------------------------------------------------------
pro lupemod::execute, label
	x = dialog_message(['Overload lupemod::execute','in ' + obj_class(self)])
	return
end

;+=----------------------------------------------------------------------------
; Name  : lupemod::reset
; Desc  : Reset the internal status of this module - i.e. ess. bring it into
;         the same state as right after instanciation
;=-----------------------------------------------------------------------------
pro lupemod::reset
	x = dialog_message(['Overload lupemod::reset','in ' + obj_class(self)])
	return
end

;+=----------------------------------------------------------------------------
; Name  : lupemod::update
; Desc  : The module makes use of data that might have been changed. Check
;         for data changes and update the internal status and display
;         accordingly.
;=-----------------------------------------------------------------------------
pro lupemod::update
	x = dialog_message(['Overload lupemod::update','in ' + obj_class(self)])
	return
end

;+=----------------------------------------------------------------------------
; Name  : lupemod::evthandler
; Desc  :
;=-----------------------------------------------------------------------------
pro lupemod::evthandler, evt
	x = dialog_message(['Overload lupemod::evthandler','in ' + obj_class(self)])
	return
end

;==============================================================================
;
;                     HANDLING OF BASE WIDGET AND EVENTS
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : lupemod::createBaseWidget
; Syntax:
; Args  :
; Desc  : This function creates a base widget for both embedded and top-level
;         modules, although the arguments differ as embedded modules have a
;         fixed size which cannot be changed.
;         Example for a top-level module:
;         res = self->createBaseWidget( /column, scr_xsize=345, scr_ysize=283, title='Prefs' )
;         Example for an embedded module:
;         res = self->createBaseWidget( /column )
;=-----------------------------------------------------------------------------
function lupemod::createBaseWidget, _ref_extra=ex
	case self.type of
		0: return, 0
		1: begin
				; create no base widget if already registered
			if( xregistered(self.name) ne 0 ) then return, 0
			if( self.baseid ne 0L and widget_info( self.baseid, /valid_id ) eq 1 ) then return, 0
				; create the base
			if( (*(*self.g).preferences).floatingWins eq 1 ) then begin
				baseid = widget_base( /tlb_move_events, _extra=ex, kill_notify='lupemod_killevt', /floating, group_leader=(*self.g).topid )
			end else begin
				baseid = widget_base( /tlb_move_events, _extra=ex, kill_notify='lupemod_killevt' )
			end
			self.baseid = baseid
			widget_control, self.baseid, set_uvalue={o:self}
			return, 1
		end
		2: begin
				; don't act, if we are already mapped
			if( widget_info( self.baseid, /valid_id ) eq 1 ) then return, 0
			self.baseid = widget_base( (*self.g).topid, xsize=1016, ysize=695, _extra=ex )
			widget_control, self.baseid, set_uvalue={o:self}
			return, 1
		end
	end
end

;+=----------------------------------------------------------------------------
; Name  : lupemod::placeAndRegister
; Desc  : If the module is a 'top' module, it is placed on screen, either at
;         the center or at the previously stored position, and then registers
;         the module with xmanager. Since object orientation has not arrived
;         yet in IDL's widget-world, this needs to be a 'static' procedure:
;         lupemod_evthandler
;=-----------------------------------------------------------------------------
pro lupemod::placeAndRegister
	case self.type of
		0: return
		1: begin
			if( self.scr_xpos eq -1 ) then begin
				geom   = widget_info( self.baseid, /geometry )
				screen = get_screen_size()
				self.scr_xpos = screen[0]/2-geom.scr_xsize/2-geom.margin
				self.scr_ypos = screen[1]/2-geom.scr_ysize/2-geom.margin
			end
			widget_control, self.baseid, xoffset=self.scr_xpos, yoffset=self.scr_ypos
		end
		2: begin
			; need to do anything special, or just register?
		end
	end
	widget_control, self.baseid, /realize
	xmanager, self.name, self.baseid, event_handler='lupemod_evthandler', /just_reg, group_leader=self.baseid
end

;+=----------------------------------------------------------------------------
; Name  : lupemod_evthandler
; Desc  : A 'static' method, so to speak. All module events arrive here -
;         the widgets have their object and the method to call stored as
;         anonymous structure in their uvalue. This event handler does
;         nothing more than calling this method.
;=-----------------------------------------------------------------------------
pro lupemod_evthandler, evt
	widget_control, evt.id, get_uvalue=call
	if( evt.top eq evt.id ) then begin ; an event from the base!
		call.o->setScrXpos, evt.x
		call.o->setScrYpos, evt.y
		return
	end
	call_method, call.m, call.o, evt
end

;+=----------------------------------------------------------------------------
; Name  : lupemod_killevt
; Desc  : This function is called, whenever a window is removed from screen,
;         whether via Alt-F4, the window's close button or the close window handle.
;         We have it here to catch unexpected kills - by the time we get here,
;         the window itself has already been removed from screen by the WM,
;         but we may still do the book-keeping.
;=-----------------------------------------------------------------------------
pro lupemod_killevt, baseid
	widget_control, baseid, get_uvalue=uval
	call_method, 'killevt', uval.o
end

;+=----------------------------------------------------------------------------
; Name  : lupemod::killevt
; Desc  : This method is called when the window has been killed via Alt-F4.
;         We call the unmap method which gives the module the chance to do some
;         cleanup. The window itself is already gone!
;         There's no need to overload this method - check for the /killed keyword
;         in unmap if anything UI specific needs to be done.
;=-----------------------------------------------------------------------------
pro lupemod::killevt
	self->unmap, /killed
end



;==============================================================================
;
;                GENERIC FUNCTIONS WHICH NEED NOT BE OVERLOADED
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : lupemod::type
; Syntax: type = obj->getType()
; Desc  : Returns the type of this module. Possible values are
;         0: if the module implements no UI functionality
;         1: if the module creates its own separate top level window
;         2: if the module is embedded in the main window
;=-----------------------------------------------------------------------------
function lupemod::type
	return, self.type
end

function lupemod::exclusive
	return, self.exclusive
end

function lupemod::name
	return, self.name
end

;+=----------------------------------------------------------------------------
; Name  : lupemod::setScrXpos
; Desc  : store the current window x-position for later use
;=-----------------------------------------------------------------------------
pro lupemod::setScrXpos, x
	self.scr_xpos = x
end

;+=----------------------------------------------------------------------------
; Name  : lupemod::setScrYpos
; Desc  : store the current window y-position for later use
;=-----------------------------------------------------------------------------
pro lupemod::setScrYpos, y
	self.scr_ypos = y
end

;+=----------------------------------------------------------------------------
; Name  : lupemod__define
; Desc  : Private Procedure (constructor)
;=-----------------------------------------------------------------------------
pro lupemod__define
	obj = { LUPEMOD					,$
			name:				''  ,$ ; the module's name
			type:				 0  ,$ ; 0: no UI, 1: top, 2: embedded
			exclusive:			 0  ,$ ; 1: yes, 0: no
			revision:			''  ,$ ; string holding the revision number
			g:			  ptr_new() ,$ ; pointer to the global structure
			baseid:				 0L	,$ ; baseid for 'top' modules
			scr_xpos:            -1 ,$ ; screen x position
			scr_ypos:            -1  $ ; screen y position
		  }
end