;+=----------------------------------------------------------------------------
; Name    : mod_aif
; RCS     : $Id: mod_aif__define.pro,v 1.4 2004/11/25 20:10:52 olivert Exp olivert $
; Desc    : manual AIF selection
;           The extracted AIF curve is saved in an mrImageSet object with
;           class='aif' and name='global', with exactly one 'image' stored
;           in the set with dimensions [number of shots, 1]. This 'image'
; Author  : (c) 2004 Oliver Thilmann
; TODO    : The parameters that steer the automatic AIF finding should be
;           set in preferences.
;=------------------------------------------------------------------------------
function mod_aif::revision
	rev = '$Revision: 1.4 $'
	return, rev
end

;+=----------------------------------------------------------------------------
; Name  : mod_aif::init
; Desc  : Embedded, non-exclusive module
;=-----------------------------------------------------------------------------
function mod_aif::init, g
    res = self->lupemod::init( 'mod_aif', 'embedded', 0, g )
    if( res eq 0 ) then return, 0

	self.dispbaseline = 1
	self.disptimepos  = 1
	self.weightlinear = 1
	self.dispsig      = 1

	return, 1
end

;+=----------------------------------------------------------------------------
; Name  : mod_aif::readyToExecute
; Desc  : ok to run, if raw data present. Concentration images can then be
;         calculated when the module is mapped.
;=-----------------------------------------------------------------------------
function mod_aif::readyToExecute, label
	i18n    = (*self.g).i18n
	imgsets = (*self.g).imgSets
	srcimgs = mrGetImageSet( imgsets, class='raw', name='signal' )
	if( srcimgs eq obj_new() ) then begin
		x = dialog_message( i18n->get('err_noPerfusionSetLoaded'), title=i18n->get('Info'), /info )  ; don't shock the user
		return, 0
	end
	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_aif::map
; Desc  : Create the UI, calculate concentration images, if not available
;=-----------------------------------------------------------------------------
pro mod_aif::map, option

	i18n = (*self.g).i18n

		; This module needs condensed signal and concentration images.
		; We also check for a baseline image set as this is part of the set,
		; and calculate any of the missing image sets here. From readyToExecute,
		; we know that a signal set is present.

	sigset = mrGetImageSet( (*self.g).imgSets, class='raw', name='signal' )
	sigtset = mrGetImageSet( (*self.g).imgSets, class='raw', name='signal_xyt' )
	ctrset = mrGetImageSet( (*self.g).imgSets, class='raw', name='ctr_xyt')
	baseset= mrgetImageSet( (*self.g).imgSets, class='raw', name='baseline')

	modCalc = (*self.g).modules->get('mod_calc')

	if( baseset eq obj_new() ) then modCalc->calcBaselineImgset, /progressbar
	if( sigtset eq obj_new() or ctrset eq obj_new() ) then modCalc->calcCtrImgsetFromT2star, /progressbar

	sigtset = mrGetImageSet( (*self.g).imgSets, class='raw', name='signal_xyt' )
	ctrset = mrGetImageSet( (*self.g).imgSets, class='raw', name='ctr_xyt')
	baseset= mrgetImageSet( (*self.g).imgSets, class='raw', name='baseline')

		; Create the base widget - this is an embedded module, so no xsize or ysize settings

	res = self->createBaseWidget( /row )
	if( res eq 0 ) then return

	nslices = sigset->nslices()
	nshots  = sigset->nshots()
	self.currentShot  = self.currentShot  < (nshots-1)
	self.currentSlice = self.currentSlice < (nslices-1)

		; widget code starts here

	b1 = widget_base( self.baseid, /column, /frame, group_leader=self.baseid )  ; left : mrdisplay & sliders
	b2 = widget_base( self.baseid, /column, /frame, group_leader=self.baseid )  ; right: title/buttons/contrast

		; b1: mrdisplay and sliders

	self.roi       = obj_new('mrroiui', b1 )
	r1 = widget_base( b1, /column, /frame )
	r2 = widget_base( r1, /row )
	self.display   = obj_new('mrdisplay', r2, 512, 512, 512, 512, interpolate=0 )
	if( nslices gt 1 ) then begin
		self.idvslider = widget_slider( r2, xsize=30, ysize=512, /vertical, value=self.currentSlice, sensitive=1, min=0, max=nslices-1, uvalue={o:self, m:'VSliderEvt'} )
	end else begin
		self.idvslider = widget_slider( r2, xsize=30, ysize=512, /vertical, value=self.currentSlice, sensitive=0, min=0, max=1)
	end
	r2 = widget_base( r1, /row )
	self.idhslider = widget_slider( r1, xsize=512, ysize=30, value=self.currentShot, sensitive=1, min=0, max=nshots-1, uvalue={o:self, m:'HSliderEvt'} )
	self.contrast  = obj_new('mrcontrast', b1, xsize=552, ysize=87 )

		; b2: drawing widets and buttons

	r1 = widget_base( b2, /align_right, xsize=430 )
    x  = widget_label( r1, value=i18n->get('aif_title'), font='fixedsys', /align_center, /sunken_frame, frame=1, xsize=425)

		; here's the two button blocks in the upper part
		; unluckily every widget_base consumes some space, framed or not - to get
		; frames on the same height, one thus has to use invisible dummy bases
		; maybe IDL one day introduces a better packer (and more beautiful widgets)

	r1 = widget_base( b2, column = 2 )
	r2 = widget_base( r1, /column )

	r3 = widget_base( r2, /column, /frame, xsize=203, ysize=100 )
	x  = widget_label ( r3, /align_left, font='fixedsys', value=i18n->get('Display')+':' )
	r4 = widget_base  ( r3, /nonexclusive, /column )
	x  = widget_button( r4, value=i18n->get('aif_baselineBoundaries'), uvalue={o:self, m:'BaselineButtonEvt'} )
	widget_control, x, set_button=self.dispbaseline
	x  = widget_button( r4, value=i18n->get('aif_currentTimePosition'), uvalue={o:self, m:'TimeposButtonEvt'} )
	widget_control, x,  set_button=self.disptimepos
	r4 = widget_base  ( r3, /exclusive, /row, ysize=20 )
	x  = widget_button( r4, value=i18n->get('Signal'), uvalue={o:self, m:'displaySignalButtonEvt'} )
	widget_control, x, set_button=self.dispsig
	self.iddispsig = x
	x  = widget_button( r4, value=i18n->get('Concentration'), uvalue={o:self, m:'displayCtrButtonEvt'} )
	widget_control, x, set_button=1-self.dispsig
	self.iddispctr = x


	r2 = widget_base( r1, /column, xsize=209 )
	r3 = widget_base( r2, /column, /frame, xsize=208 )
	x  = widget_label ( r3, /align_left, font='fixedsys', value=i18n->get('aif_averaging') )
	r4 = widget_base  ( r3, /exclusive, /row, ysize=20 )
	x  = widget_button( r4, value=i18n->get('Linear'),   uvalue={o:self, m:'weightLinearButtonEvt'} )
	widget_control, x, set_button=self.weightlinear
	x  = widget_button( r4, value=i18n->get('Weighted'), uvalue={o:self, m:'weightWeightedButtonEvt'} )
	widget_control, x, set_button=1-self.weightlinear
	r3 = widget_base( r2, /column, xsize=204  )
	x  = widget_button( r3, value=i18n->get('aif_suggest'), uvalue={o:self, m:'suggestAifButtonEvt'} )
	x  = widget_button( r3, value=i18n->get('aif_deleteAllAifPixels'), uvalue={o:self, m:'deleteAllButtonEvt'} )

		; followed by the 6 drawing widgets

	r1 = widget_base( b2, /row )
	self.idsig1 = widget_draw( r1, scr_xsize=211, scr_ysize=162 )
	self.idctr1 = widget_draw( r1, scr_xsize=211, scr_ysize=162 )

	r1 = widget_base( b2, /row )
	self.idsig2 = widget_draw( r1, scr_xsize=211, scr_ysize=162 )
	self.idctr2 = widget_draw( r1, scr_xsize=211, scr_ysize=162 )

	r1 = widget_base( b2, /row )
	self.idsig3 = widget_draw( r1, scr_xsize=211, scr_ysize=162 )
	self.idctr3 = widget_draw( r1, scr_xsize=211, scr_ysize=162 )

	r1 = widget_base  ( b2, /column, xsize=217, /align_right )
	x  = widget_button( r1, value=i18n->get('CloseModule'), /frame, uvalue={o:self, m:'closeButtonEvt'} )

		; realize the widget and register with the xmanager

	self->placeAndRegister

		; register contrast and activate display

	self->display
	self.zoom = obj_new('mrzoom')
	self.pan  = obj_new('mrpan')
	self.cursorInfo = obj_new('mrCursorInfo')
	self.contrast->selfregister, self.display, button=3, /bordermode
	self.cursorInfo->selfregister, self.display, toggleKey='c'
	self.cursorInfo->hide
	self.zoom->selfregister, self.display, button=2, /border
	self.pan->selfregister, self.display, button=2, /noborder
	self.display->activate

		; Handle roi issues:
		; first, we test whether the number of slices has changed -
		; in such case we delete all roigroups. Otherwise we use the
		; existing ones, i.e rois remain, even if the underlying data
		; has changed as long as the new data has the same number of
		; slices
		; (one could probably argue about this one, but most users won't
		; notice the varying behaviour, I guess)

	if( ptr_valid( self.roigroups ) ) then begin
		if( n_elements(*self.roigroups) ne nslices ) then begin ; hoppla, nslices has changed
			for i = 0, n_elements(*self.roigroups)-1 do begin
				obj_destroy, (*self.roigroups)[i]
			end
			ptr_free, self.roigroups
			self.roigroups = ptr_new( objarr(nslices), /no_copy )
		end
	end else begin
		self.roigroups = ptr_new( objarr(nslices), /no_copy )
	end

		; override mrroi's load and save methods - we want to save ROIs from all slices

	self.roi->setLoadMethod, self, 'load'
	self.roi->setSaveMethod, self, 'save'
	self.roi->selfregister, self.display, button=1
	self.roi->activate
	if( obj_valid ((*self.roigroups)[self.currentSlice] )) then begin
		self.roi->setRoiGroup, (*self.roigroups)[self.currentSlice]
		self->evaluateRois     ; this will also update the AIF
		self.display->display
	end

		; register with display and roi event for callback

	self.display->registerEvt, self, ['Motion', 'KeyPress']
	self.roi->notify,          self, 'mrRoiEvtHandler', ['modify','change']

		; draw empty frames to start with or, if an AIF had been defined,
		; evaluate the ROIs

	self->evaluateRois

		; we make it simple for us here: all changes to image sets used in
		; this module will make the module unmap
		; A simple check in the evaluation method is not sufficient as (for
		; whatever nasty reason) the number of slices might have changed.

		; There's a problem here as mrNotify->sendNotification is not re-entrant.
		; So for now we only check the image sets themselves...

;	(*self.g).imgSets->notify, self, 'imgSetChange', ['add','remove']
	sigset->notify, self, 'imgSetChange', ['set','delete']
	ctrset->notify, self, 'imgSetChange', ['set','delete']
	sigtset->notify, self, 'imgSetChange', ['set','delete']
	baseset->notify, self, 'imgSetChange', ['set','delete']

end


;+=----------------------------------------------------------------------------
; Name  : mod_aif::unmap
; Args  : <b>killed</b>
;         If the user killed the window via Ctrl-F4 or the close handle (the
;         cross at the upper right), killed is set to '1'. In this case it
;         might be (I have not tested this) that the system's window manager
;         has already removed the window from screen.
; Desc  : Remove the window from screen
;=-----------------------------------------------------------------------------
pro mod_aif::unmap, _ref_extra = ex
	obj_destroy, self.display
	obj_destroy, self.roi
	obj_destroy, self.contrast
	obj_destroy, self.zoom
	obj_destroy, self.pan
	obj_destroy, self.cursorInfo

		; clear notifications - this time we need to check whether the sets exist

;	(*self.g).imgSets->notify, self, /remove
	set = mrGetImageSet( (*self.g).imgSets, class='raw', name='signal' )
	if( obj_valid(set) ) then set->notify, self, /remove
	set = mrGetImageSet( (*self.g).imgSets, class='raw', name='signal_xyt' )
	if( obj_valid(set) ) then set->notify, self, /remove
	set = mrGetImageSet( (*self.g).imgSets, class='raw', name='ctr_xyt')
	if( obj_valid(set) ) then set->notify, self, /remove
	set= mrgetImageSet( (*self.g).imgSets, class='raw', name='baseline')
	if( obj_valid(set) ) then set->notify, self, /remove

	self->lupemod::unmap, _strict_extra = ex
end


;==============================================================================
;
;                           E V E N T   H A N D L I N G
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mod_aif::VSliderEvt
; Desc  : handles events from the vertical slider
;=-----------------------------------------------------------------------------
pro mod_aif::VSliderEvt, evt
	self.currentSlice = evt.value
	self.roi->disableNotify
	self.roi->setRoiGroup, (*self.roigroups)[self.currentSlice]
	self.roi->enableNotify
	self->evaluateRois
	self->display, /update
end

;+=----------------------------------------------------------------------------
; Name  : mod_aif::HSliderEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_aif::HSliderEvt, evt
	self.currentShot = evt.value
	self->display, /update
end

;+=----------------------------------------------------------------------------
; Name  : mod_aif::BaselineButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_aif::BaselineButtonEvt, evt
	self.dispbaseline = evt.select
	x = ptr_new( {imgx:0, imgy:0, key:0L}, /no_copy )
	self->mrDisplayEvtHandler, x
	ptr_free, x
end

;+=----------------------------------------------------------------------------
; Name  : mod_aif::TimeposButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_aif::timeposButtonEvt, evt
	self.disptimepos = evt.select
	x = ptr_new( {imgx:0, imgy:0, key:0L}, /no_copy )
	self->mrDisplayEvtHandler, x
	ptr_free, x
end

;+=----------------------------------------------------------------------------
; Name  : mod_aif::suggestAifButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_aif::suggestAifButtonEvt, evt
	self->suggestAifPixels
end

;+=----------------------------------------------------------------------------
; Name  : mod_aif::displaySignalButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_aif::displaySignalButtonEvt, evt
	self.dispsig = evt.select
	if( self.dispsig eq 0 and mrGetImageSet( (*self.g).imgSets, class='raw', name='ctr_xyt' ) eq obj_new() ) then begin
		self->calcConcentrationImages
	end
	self.display->setGamma, 1.
	self.display->setContrast, .5, 1.
	self->display
end
pro mod_aif::displayCtrButtonEvt, evt
	; dummy, taken care of by above method
end

;+=----------------------------------------------------------------------------
; Name  : mod_aif::weightLinearButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_aif::weightLinearButtonEvt, evt
	self.weightlinear = evt.select
	self->evaluateRois
end
pro mod_aif::weightWeightedButtonEvt, evt
end

;+=----------------------------------------------------------------------------
; Name  : mod_aif::deleteAllButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_aif::deleteAllButtonEvt, evt
	i18n = (*self.g).i18n
	x = mrdialog_select([i18n->get('aif_deleteConfirmation1'),i18n->get('aif_deleteConfirmation2')],$
	                    [i18n->get('Yes'),i18n->get('No')], title=i18n->get('Question'), $
						group_leader=self.baseid, default=1, bsize=80, /question )
	if( x eq 0 ) then self->deleteAllRois
end

;+=----------------------------------------------------------------------------
; Name  : mod_aif::closeButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_aif::closeButtonEvt, evt
	self->unmap
end


;==============================================================================
;
;                                 I N T E R N A L
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mod_aif::display
; Desc  : Display an image in the mrdisplay object, according to the current
;         values of dispsig (signal or concentration iamge?), currentSlice and
;         currentShot
;         Contrast and gamma settings are preserved
;         (This could also be handled via the update method, but then we'd need
;         to make the first call to this function a special case - this way
;         it's easier)
;=-----------------------------------------------------------------------------
pro mod_aif::display, update=update

	update = keyword_set(update)

	case self.dispsig of
		0: imgset = mrGetImageSet( (*self.g).imgSets, class='raw', name='ctr_xyt' )
		1: imgset = mrGetImageSet( (*self.g).imgSets, class='raw', name='signal_xyt' )
	end

	img = intarr( imgset->xsize(), imgset->ysize() )
	img[*,*] = (*(*(imgset->imgData()))[self.currentSlice])[*,*,self.currentShot]


	self.display->setImg, ptr_new( img, /no_copy ), /no_copy, update=update
	if( not update ) then self.display->fitToScreen
	self.display->display

end

;+=----------------------------------------------------------------------------
; Name  : mod_aif::deleteAllRois
; Desc  : Delete all roigroups, i.e. all defined ROIs in all slices
;=-----------------------------------------------------------------------------
pro mod_aif::deleteAllRois

	; Here's a nasty way to shoot yourself in the foot:
	; If we first set the roigroup to obj_new (which will implicitly create
	; a new and empty one), the subsequent call to evaluateRois from the
	; callback will add this newly created roi to our internal list, which
	; we then destroy....
	; So rather do it vice-versa

	for i = 0, n_elements( *self.roigroups )-1 do begin
		obj_destroy, (*self.roigroups)[i]
		(*self.roigroups)[i] = obj_new()
	end
	self.roi->setRoiGroup, obj_new()         ; causes callback to evaluateRois
	self.display->display
end


;+=----------------------------------------------------------------------------
; Name  : mod_aif::evaluateRois
; Syntax: obj->evaluateRois, /no_global
; Args  : <b>no_global</b>
;         If set, the global AIF functions are not calculated.
;         This is used for roiupdate events, to speed things up.
; Desc  : Update the signal and concentration curves for the current slice and
;         (unless no_global is set) for all slices - the latter thereby also
;         calculates the current AIF and stores it in an mrImageSet object.
;         If the AIF is zilch (because no ROIs are defined), a possibly existing
;         mrImageSet object is destroyed.
;=-----------------------------------------------------------------------------
pro mod_aif::evaluateRois, no_global=no_global

	i18n = (*self.g).i18n

	no_global = keyword_set( no_global )

	imgsig   = mrGetImageSet( (*self.g).imgSets, class='raw', name='signal' )
	imgctr   = mrGetImageSet( (*self.g).imgSets, class='raw', name='ctr_xyt' )
	roigroup = self.roi->roiGroup()

	nshots  = imgsig->nshots()
	nslices = imgsig->nslices()

	(*self.roigroups)[self.currentSlice] = roigroup

	nrois = 0
	for i = 0, n_elements(*self.roigroups)-1 do begin
		if( obj_valid( (*self.roigroups)[i] ) ) then nrois = nrois + (*self.roigroups)[i]->count()
	end
	nrois_curr = roigroup->count()

       ; no rois in current slice? -> blank widgets

    if( nrois_curr eq 0 ) then begin
		widget_control, self.idsig2, get_value=win
		wset, win
		plot, [0,0], yrange=[0,1000], xrange=[0, nshots], title=i18n->get('aif_thisSlice')
		widget_control, self.idctr2, get_value=win
		wset, win
		plot, [0,0], yrange=[0,1000], xrange=[0, nshots], title=i18n->get('aif_thisSlice')
    end

       ; no rois at all? -> blank total AIF, delete aif mrImageSet object and return

    if( nrois eq 0 ) then begin
	    widget_control, self.idsig3, get_value=win
	    wset, win
	    plot, [0,0], yrange=[0,1000], xrange=[0, nshots], title=i18n->get('aif_noRoisDefined')
	    widget_control, self.idctr3, get_value=win
	    wset, win
	    plot, [0,0], yrange=[0,1000], xrange=[0, nshots], title=i18n->get('aif_noRoisDefined')

		aifset = mrGetImageSet( (*self.g).imgSets, class='aif', name='global' )
		if( aifset ne obj_new() ) then begin
			(*self.g).imgSets->remove, aifset
			obj_destroy, aifset
		end
	    return
    end

    sig        = dblarr( nrois, nshots )                        ; signal goes in here
    ctr        = dblarr( nrois, nshots )

       ; FIRST: evalute this slice's ROIs...

    if( nrois_curr gt 0 ) then begin                            ; eval rois of current slice
		roino      = intarr( nrois_curr )               ; rois are not necessarily sorted by number
		for i = 0, nrois_curr-1 do begin
			roi          = roigroup->get( position=i )
			roino[i]     = roi->id()
			if( roino[i] eq 999 ) then roino[i] = 0
			mask         = where( *(roi->filledMask()) gt 0, npixels )

			if( npixels eq 0 ) then begin
				sig[i,*] = 0
				continue
			end

			for j = 0, nshots-1 do begin
				sig[i,j] = mean( (*(*(imgsig->imgData()))[self.currentSlice, j])[mask] )
				ctr[i,j] = mean( ((*(*(imgctr->imgData()))[self.currentSlice])[*,*,j])[mask] )
			end
		end

	end

		; ... and display them

	x = self.roi->palette( red=red, green=green, blue=blue )

	widget_control, self.idsig2, get_value=win
	wset, win
	tvlct, red, green, blue
	m = max( sig )
	plot, [0,0], yrange=[0,m], xrange=[0,nshots], color=255, title=i18n->get('aif_thisSlice')
	for i = 0, nrois_curr-1 do oplot, sig[i,*], color=roino[i]+1

	widget_control, self.idctr2, get_value=win
	wset, win
	tvlct, red, green, blue
	m = max( ctr )
	plot, [0,0], yrange=[0,m], xrange=[0,nshots], color=255, title=i18n->get('aif_thisSlice')
	for i = 0, nrois_curr-1 do oplot, ctr[i,*], color=roino[i]+1

		; roiupdate events return here: the calculation of the total AIF can take
		; quite some time if many ROIs in many slices are defined - a pain on slow
		; machines as this tremendously slows down creation/modification of rois.
		; Thus we calculate the AIF only for ROIChange events, i.e. when the mouse button
		; has been released and the ROI defined

	if( no_global ) then return

		; OK, the ROI is fixed. Now we calculate overall AIF, display and save it
		; (admittedly a bit of copy'n paste, but what the heck)

	sig[0,*]  = 0.   ; we use the first sig and ctr to store the mean values
	ctr[0,*]  = 0.
	totrois   = 0
	totpixels = 0

	for sliceno = 0, nslices-1 do begin
		roigroup = (*self.roigroups)[sliceno]
		if( not obj_valid( roigroup )) then continue
		nrois    = roigroup->count()
		if( nrois eq 0 ) then continue
		for j = 0, nrois-1 do begin
			roi       = roigroup->get( position=j )
			mask      = where( *(roi->filledMask()) gt 0, npixels )
			if( mask[0] eq -1 ) then continue

			if( self.weightlinear eq 1 ) then npixels=1
			for k = 0, nshots-1 do begin
				sig[0,k] = temporary(sig[0,k]) + 1. * npixels * mean( (*(*(imgsig->imgData()))[sliceno,k])[mask] )
				ctr[0,k] = temporary(ctr[0,k]) + 1. * npixels * mean( ((*(*(imgctr->imgData()))[sliceno])[*,*,k])[mask] )
			end
			totpixels = totpixels+npixels
			totrois   = totrois+1
		end
	end
	sig[0,*] = temporary(sig[0,*])/totpixels
	ctr[0,*] = temporary(ctr[0,*])/totpixels

       ; finally plot the AIF ...

	widget_control, self.idsig3, get_value=win
	wset, win
	tvlct, red, green, blue
	m = max( sig[0,*] )
	plot, [0,0], yrange=[0,m], xrange=[0,nshots], color=255, title=i18n->get('aif_signalAllSlices')
	oplot, sig[0,*], color=4

    if( totrois eq 1 ) then title=i18n->get('aif_fromOneRoi') else title = string( totrois ,format=i18n->get('aif_fromNRois')) ; !@$$# perfectionism
	widget_control, self.idctr3, get_value=win
	wset, win
	tvlct, red, green, blue
	m = max( ctr[0,*] )
	plot, [0,0], yrange=[0,m], xrange=[0,nshots], color=255, title=title
	oplot, ctr[0,*], color=4

		; store the result in an mrImageSet object
		; check whether image set exists and create a new one, if not

	aifset = mrGetImageSet( (*self.g).imgsets, class='aif', name='global' )
	if( aifset eq obj_new() ) then begin
		aifset = obj_new('mrImageSet', 1, class='aif', name='global', xsize=nshots, ysize=1 )
		(*self.g).imgsets->add, aifset
	end

		; check whether the 'image' exists and has the right dimension
		; create new one, if not

	aif = aifset->imgData()
	if( ptr_valid( (*aif)[0] ) ) then begin
		if( n_elements( *(*aif)[0] ) ne nshots ) then begin
			ptr_free, (*aif)[0]
			(*aif)[0] = ptr_new( fltarr(nshots), /no_copy )
		end
	end else begin
		(*aif)[0] = ptr_new( fltarr(nshots), /no_copy )
	end
	(*(*aif)[0])[*] = ctr[0,*]
end

;==============================================================================
;
;                             NON-UI RELATED METHODS
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : mod_aif::load
; Desc  : Hooked into mrroi's load button - as opposed to mrroi's build-in
;         load function, this one is able to load a roigroup for each slice.
;=-----------------------------------------------------------------------------
pro mod_aif::load, evt

	i18n = (*self.g).i18n

	nrois = 0
	for i = 0, n_elements( *self.roigroups )-1 do begin
		if( not obj_valid( (*self.roigroups)[i] )) then continue
		nrois = nrois + (*self.roigroups)[i]->count()
	end

	if( nrois gt 0 ) then begin
		x = mrdialog_select([i18n->get('aif_loadWarning'),i18n->get('Continue')+'?'], $
		                    [i18n->get('Yes'),i18n->get('No')], default=1, group_leader=self.baseid, $
		                    title=i18n->get('Question'), /warning )
		if( x eq 1 ) then return
	end

	file = dialog_pickfile( /read, title=i18n->get('aif_selectAifFile'), path=(*self.g).workingdir, get_path=path, /must_exist )
	if( file eq '' ) then return

	(*self.g).workingdir = path
	roigroups = mrRoiGroup_load( file )
	if( roigroups eq obj_new() ) then begin
		x = dialog_message( i18n->get('err_readingFile'), title=i18n->get('Error'), /error )
		return
	end

	self->deleteAllRois

	nslices = n_elements( *self.roigroups )
	nrois = roigroups->count()
	for i = 0, nrois-1 do begin
		roigroup = roigroups->get( position=0 )
		id = roigroup->id()
		if( id ge nslices ) then begin
			x = dialog_message( i18n->get('aif_roigroupSliceOutOfBounds') + ': ' + strtrim(id, 2), title=i18n->get('Error'), /error )
			roigroups->remove, roigroup
			obj_destroy, roigroup
		end	else begin
			(*self.roigroups)[id] = roigroup
			roigroups->remove, roigroup
		end
	end

	obj_destroy, roigroups

	self.roi->setRoigroup, (*self.roigroups)[self.currentSlice]

	self->evaluateRois
	self.display->display

end


;+=----------------------------------------------------------------------------
; Name  : mod_aif::save
; Desc  : Hooked into mrroi's save button. As opposed to mrroi's built-in save
;         function this one here saves all roigroups from all slices
;=-----------------------------------------------------------------------------
pro mod_aif::save, evt

	i18n = (*self.g).i18n

	nrois = 0
	for i = 0, n_elements( *self.roigroups )-1 do begin
		if( not obj_valid( (*self.roigroups)[i] )) then continue
		nrois = nrois + (*self.roigroups)[i]->count()
	end

	if( nrois eq 0 ) then begin
		x = dialog_message( i18n->get('aif_noRoisToSave'), title=i18n->get('Info'), /info)
		return
	end

	file = dialog_pickfile( /write, title= i18n->get('aif_selectAifFileForWrite'), path=(*self.g).workingdir, get_path=path )
	if( file eq '' ) then return

	(*self.g).workingdir = path
	unit = mrRoiGroup_openxml( file )
	if( unit eq 0 ) then begin
		x = dialog_message( i18n->get('err_openFile'), title=i18n->get('Error'), /error )
		return
	end
	for i = 0, n_elements( *self.roigroups )-1 do begin
		roigroup = (*self.roigroups)[i]
		if( roigroup eq obj_new() ) then continue
		if( roigroup->count() eq 0 ) then continue
		roigroup->setId, i                         ; the slice number
		roigroup->write, unit
	end
	mrroigroup_closexml, unit
	x = dialog_message( i18n->get('aif_saveConfirmation'), title=i18n->get('Info'), /info)
end



;+=----------------------------------------------------------------------------
; Name  : mod_aif::suggestAIFpixels
; Desc  : suggests 10 AIF pixels in the currently selected slice
;         Criteria are:
;         - baseline at least 60% of maximum
;         - relative signal drop at least 80%
;         - signal must come back to at least 80% of original baseline level
;         - continuous increase and decrease aroung the maximum concentration
;           value down to 20% of maximum concentration
;         - maximum must be early in time (the penalty for late maximum is
;           very heavy - curves with a maxium 2 timebins after the earliest
;           one stand no chance to get into the top 10)
;=-----------------------------------------------------------------------------
pro mod_aif::suggestAIFpixels

	imgsig = mrGetImageSet( (*self.g).imgSets, class='raw', name='signal' )
	sigxyt = mrGetImageSet( (*self.g).imgSets, class='raw', name='signal_xyt' )
	ctrxyt = mrGetImageSet( (*self.g).imgSets, class='raw', name='ctr_xyt' )
	base   = mrGetImageSet( (*self.g).imgSets, class='raw', name='baseline' )


    xsize  = imgsig->xsize()
    ysize  = imgsig->ysize()
    nshots = imgsig->nshots()
    sig    = (*(sigxyt->imgData()))[self.currentSlice]
    ctr    = (*(ctrxyt->imgData()))[self.currentSlice]
    s0     = (*(base->imgData()))[self.currentSlice]
    s0min  = 0.6 * max( *s0 )                               ; baseline at least 60% of max

       ; a structure would be nicer, but this is simpler

    aifx      = intarr( 500 )   ; x position of the aif
    aify      = intarr( 500 )   ; y position of the aif
    aifpeak   = fltarr( 500 )   ; maximum concentration value
    aiftime   = intarr( 500 )   ; index of maximum
    aifarea   = intarr( 500 )   ; a rough area estimation
    aifweight = intarr( 500 )   ; for later weighting

    naifs      = 0
    aiftime[*] = 100

    for x = 0, xsize-1 do begin
       for y = 0, ysize-1 do begin

         area = 0

          ; baseline must be at least 60% of maximum
         if( (*s0)[x,y] lt s0min ) then continue                    ; minimum baseline

          ; signal drop must be at least to 20% of baseline
         if( min((*sig)[x,y,*]) gt 0.20*(*s0)[x,y] ) then continue  ; minimum signal drop

          ; end must not be less than 80% of baseline
         endmean = mean( (*sig)[x,y,nshots-10:nshots-1] )
         if( endmean lt 0.80 * (*s0)[x,y] ) then continue

         maxctr = max( (*ctr)[x,y,*], pos )

         for i = pos+1, nshots-2 do begin                           ; continuous decrease?
          area = area + (*ctr)[x,y,i]
          if( (*ctr)[x,y,i] lt .3*maxctr ) then break
          if( (*ctr)[x,y,i+1] lt (*ctr)[x,y,i] ) then continue
          goto, NEXTAIF
         end
         for i = pos, 1, -1 do begin                                ; continuous increase?
          area = area + (*ctr)[x,y,i]
          if( (*ctr)[x,y,i] lt .3*maxctr ) then break
          if( (*ctr)[x,y,i] gt (*ctr)[x,y,i-1] ) then continue
          goto, NEXTAIF
         end
         aifx[naifs]     = x
         aify[naifs]     = y
         aifpeak[naifs]  = maxctr
         aiftime[naifs]  = pos
         aifarea[naifs]  = area

         naifs = naifs + 1
         if( naifs ge 499 ) then goto, ENDAIF
         NEXTAIF:
       end
    end

    ENDAIF:

       ; now find the n best in our list

    n = 10 < naifs

       ; no compromises with mintime
       ; everything more than 3 ticks away is virtually excluded by the offset of - 100

    mintime = min( aiftime )
    a = where( aiftime gt mintime+2 )
    if( a[0] ne -1 ) then aifweight[a] = aifweight[a] - 100
    for i = 0, 2 do begin
       a = where( aiftime eq mintime+i )
       if( a[0] ne -1 ) then aifweight[a] = aifweight[a] - i * 10
    end

    a = reverse( sort( aifpeak ) )
    for i = 0, n-1 do aifweight[a[i]] = aifweight[a[i]] + n - i

    a = reverse( sort( aifarea ) )
    for i = 0, n-1 do aifweight[a[i]] = aifweight[a[i]] + n - i

    best = reverse( sort( aifweight ) )

    for i = 0, n-1 do begin
       self.roi->addroi, 'dot', aifx[best[i]], aify[best[i]], /no_update
    end
    self.roi->calcAreaImg
    self.display->display
end

;+=----------------------------------------------------------------------------
; Name  : mod_aif::calcConcentrationImages
; Desc  : calls the corresponding function from the calc module
;=-----------------------------------------------------------------------------
pro mod_aif::calcConcentrationImages
	modcalc = (*self.g).modules->get('mod_calc')
	modcalc->calcConcentrationImages, progressbar=1
end



;==============================================================================
;
;          NOTIFICATIONS from the ROI definitions or the image sets
;
;==============================================================================



;+=----------------------------------------------------------------------------
; Name  : mod_aif::mrroiEvtHandler
; Desc  : only calls the Roi evaluation method
;=-----------------------------------------------------------------------------
pro mod_aif::mrroiEvtHandler, source, evttype, evt
	case (*evt).type of
		1: self->evaluateRois, /no_global
		2: self->evaluateRois
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_aif::mrdisplayEvtHandler
; Desc  : On a motion event redisplay the signal and contrast curve in the
;         current pixel. We use a fixed scale in the concentration
;         display as this gives a better impression of the peak's absolute
;         size.
;=-----------------------------------------------------------------------------
pro mod_aif::mrdisplayEvtHandler, evt

    if( (*evt).imgx eq -1 or (*evt).imgy eq -1 ) then return ; do nuttin when outside

	i18n = (*self.g).i18n

    	; get time data for this point and display it in draw1

	sigset = mrGetImageSet( (*self.g).imgSets, class='raw', name='signal' )
	sigxyt = mrGetImageSet( (*self.g).imgSets, class='raw', name='signal_xyt' )
	ctrxyt = mrGetImageSet( (*self.g).imgSets, class='raw', name='ctr_xyt' )
	base   = mrGetImageSet( (*self.g).imgSets, class='raw', name='baseline' )

	nslices = sigset->nslices()
	nshots  = sigset->nshots()
	bl      = base->uvalue()

		; Dunno about key events under UNIX, but we might never run
		; under UNIX anyway since a couple of functions seem to be missing :-(

    if( (*evt).key ne 0L ) then begin

		case (*evt).key of
	    	5: self.currentShot = ( self.currentShot-1 ) > 0
	    	6: self.currentShot = ( self.currentShot+1 ) < (nShots-1)
	    	7: begin
	    		self.currentSlice = ( self.currentSlice + 1 ) < (nSlices-1)
				self.roi->disableNotify
				self.roi->setRoiGroup, (*self.roigroups)[self.currentSlice]
				self.roi->enableNotify
				self->evaluateRois
	    	end
	    	8: begin ; down
	    		self.currentSlice = ( self.currentSlice - 1 ) > 0
				self.roi->disableNotify
				self.roi->setRoiGroup, (*self.roigroups)[self.currentSlice]
				self.roi->enableNotify
				self->evaluateRois
	    	end
	    	115: begin   			; 's' as in 'signal'
	    		self.dispsig = 1 - self.dispsig
	    		self.display->setContrast, .5, 1.
	    		widget_control, self.iddispsig, set_button=self.dispsig
	    		widget_control, self.iddispctr, set_button=(1-self.dispsig)
	    	end
	    	else:
    	end
    	widget_control, self.idvslider, set_value=self.currentSlice
    	widget_control, self.idhslider, set_value=self.currentShot

		case self.dispsig of
			0: imgset = mrGetImageSet( (*self.g).imgSets, class='raw', name='ctr_xyt' )
			1: imgset = mrGetImageSet( (*self.g).imgSets, class='raw', name='signal_xyt' )
		end
	 	img = intarr( imgset->xsize(), imgset->ysize() )
		img[*,*] = (*(*(imgset->imgData()))[self.currentSlice])[*,*,self.currentShot]
		self.display->setImg, ptr_new( img, /no_copy ), /no_copy, /update
   		self.display->requestDisplay
    end

		; display signal curve under cursor

	col = indgen(256)
    vals = (*(*(sigxyt->imgdata()))[self.currentSlice])[(*evt).imgx, (*evt).imgy, *]
	max  = max((*(*(sigxyt->imgdata()))[self.currentSlice])[*, *, *])
	widget_control, self.idsig1, get_value=win
	wset, win
	tvlct, col, col, col
	plot, vals, yrange=[0, max], title=i18n->get('Signal'), psym=0, symsize=.5
	oplot, vals

	if( self.dispbaseline ) then begin
		plots, [(*bl)[0],(*bl)[0]], [0, max]
		plots, [(*bl)[1],(*bl)[1]], [0, max]
	end
	if( self.disptimepos ) then begin
		plots, self.currentShot, vals[self.currentShot], psym=4, symsize=1
	end

		; display concentration curve at cursor position

    vals = (*(*(ctrxyt->imgdata()))[self.currentSlice])[(*evt).imgx, (*evt).imgy, *]
    max  = (*(*self.g).preferences).ssvd_ctrthres*1.1
	widget_control, self.idctr1, get_value=win
	wset, win
	plot, vals, yrange=[0, max], title=i18n->get('Concentration'), psym=0, symsize=.5
	oplot, vals

	if( self.dispbaseline ) then begin
		plots, [(*bl)[0],(*bl)[0]], [0, max]
		plots, [(*bl)[1],(*bl)[1]], [0, max]
	end
	if( self.disptimepos ) then begin
		plots, self.currentShot, vals[self.currentShot], psym=4, symsize=1
	end

	return

end

;+=----------------------------------------------------------------------------
; Name  : mod_aif::imgSetChange
; Desc  : Any change in the image sets will make this module unmap.
;         This could be handled more elegantly, of course, but the benefit
;         is not worth the effort, imho.
;=-----------------------------------------------------------------------------
pro mod_aif::imgSetChange, caller, evt
	i18n = (*self.g).i18n
	x = dialog_message([ i18n->get('aif_imgSetChange1'),i18n->get('aif_imgSetChange2'),$
	                     i18n->get('aif_imgSetChange3'),i18n->get('aif_imgSetChange4')], $
	                     title=i18n->get('Info'), /info )
	self->unmap
end


;+=----------------------------------------------------------------------------
; Name  : mod_aif__define
; Desc  : Private Procedure (constructor)
;=-----------------------------------------------------------------------------
pro mod_aif__define
	obj = { MOD_aif					   ,$
			inherits LUPEMOD		   ,$
			display:		 obj_new() ,$
			contrast:		 obj_new() ,$
			cursorinfo:		 obj_new() ,$
			zoom:			 obj_new() ,$
			pan:			 obj_new() ,$
			roi:			 obj_new() ,$
			roigroups:		 ptr_new() ,$ ; save the roigroups for each slice in an objarr
			idsig1:					0L ,$
			idctr1:					0L ,$
			idsig2:					0L ,$
			idctr2:					0L ,$
			idsig3:					0L ,$
			idctr3:					0L ,$
			idhslider:              0L ,$
			idvslider:              0L ,$
			idclose:				0L ,$
			iddispsig:				0L ,$
			iddispctr:				0L ,$
			dispbaseline:			 0 ,$ ; now for the corresponding flags... baseline
			disptimepos:			 0 ,$ ; display the time position
			weightlinear:			 0 ,$ ; linear or weighted averaging?
			currentShot:			 0 ,$ ; currently displayed shot
			currentSlice:            0 ,$ ; currently displayed slice
			dispsig:			     0  $ ; display signal or concentration imgs?
		  }
end