;+=----------------------------------------------------------------------------
; Name    : mod_loadperfseq
; RCS     : $Id: mod_loadperfseq__define.pro,v 1.1 2004/11/11 11:33:03 olivert Exp olivert $
; Desc    : Load an MR DSC or a CT acquired perfusion sequence
; Author  : (c) 2004 Oliver Thilmann
;=------------------------------------------------------------------------------
function mod_loadperfseq::revision
	rev = '$Revision: 1.1 $'
	return, rev
end

;+=----------------------------------------------------------------------------
; Name  : mod_loadperfseq::init
; Syntax: obj = obj_new('lupemod', type, exclusive )
; Args  : <b>type</b>
;         One of 'noUI', 'top', 'embedded' (case-insensitive)
; Desc  :
;=-----------------------------------------------------------------------------
function mod_loadperfseq::init, g

		; call superclass constructor

    res = self->lupemod::init( 'mod_loadperfseq', 'noui', 0, g )
    if( res eq 0 ) then return, 0

	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_loadperfseq::readyToExecute
; Desc  : Loading a perfusion image set will destroy all data in the
;         perfusion_images container, that includes the original data set,
;         the concentration images and all condensed data sets.
;         If the perfusion_images container is not empty, we ask the user
;         whether to load the new data or not.
;         Then there's also the issue of having only data of one single studyUID
;         in memory, but this we do not know until the data is loaded. So
;         we need to postpone the question what to do if sutdyUIDs differ until
;         then (see the execute method)
;         Of course, if loading should fail, nothing will be changed, but again
;         that's something the execute method has to take care of.
;=-----------------------------------------------------------------------------
function mod_loadperfseq::readyToExecute, option
	g = *self.g

		; if other perfusion data is in memory, ask the user whether to
		; discard that or not

	sets = mrGetImageSet( g.imgSets, class='raw', count=nsets )
	if( nsets gt 0 ) then begin
		x = mrdialog_select([g.i18n->get('lps_willDestroyAllData1'), g.i18n->get('lps_willDestroyAllData2')], $
	 					[g.i18n->get('Yes'), g.i18n->get('No')], /question, group_leader=g.topid, title=g.i18n->get('Question'))
		return, 1-x
	end
	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_loadperfseq::execute
; Desc  : Workflow:
;         Load a new set of perfusion raw images.
;         If there's any error while loading, we tell the user and nothing
;         will be changed.
;         If the user requested to have only one studyUID in memory, we check
;         for it and, in case the studyUIDs don't match, ask the user what to
;         do, keep the old or the new data.
;         Old: destroy loaded data and bail out
;         New: destroy all data in morph_images and result_images
;              and keep on going
;         Clear the perfusion_images container and destroy all data therein.
;         Add the loaded data to the perfusion_images container. If the user
;         requested automatic calculation of the concentration images, do that
;         now (this will also create a 3-dim condensed data set with dimensions
;         [x,y,t]
;         Then we're done. A brand new set of perfusion data in memory, and
;         up to three data objects in perfusion_images: original signal,
;         concentration, and concentration condensed.
;=-----------------------------------------------------------------------------
pro mod_loadperfseq::execute, option

	g     = self.g              ; pointer to global structure
	prefs = (*g).preferences    ; pointer to preferences structure
	i18n  = (*g).i18n

		; select a file, instanciate the object and load the images

	file = dialog_pickfile( path = (*g).workingdir, get_path=path, /must_exist, /read, title=i18n->get('lps_selectFile') )
	if( file eq '' ) then return
	newset   = obj_new('mrimagesetdicom', class='raw', name='signal')
	res      = newset->loadTimeSequence( file, /progressbar, /rotate )
	if( res eq 0 ) then begin
		errtxt = newset->errMsg()
		res = dialog_message([i18n->get('err_loadingSequence')+':', errtxt], title=i18n->get('Error'), /error)
		obj_destroy, newset
		return
	end

		; check whether this image set looks like a perfusion set
		; The limit of 10 shots is of course rather arbitrary but (hopefully) reasonable

	if( newset->nshots() lt 10 ) then begin
		line1 = string( newset->nshots(), format=i18n->get('lps_tooFewTimepoints1') )
		res = mrdialog_select([ line1, i18n->get('lps_tooFewTimepoints2')], i18n->get('lps_tooBad'), $
		                       group_leader=(*g).topid, /skull, title=i18n->get('lps_suspiciousDataLayout') )
		obj_destroy, newset
		return
	end

		; check whether the study UID of the loaded data matches the current
		; studyUID - if so, the user has to chose which data to keep
		; If he decides to keep the new data destroy all other image sets

	if( (*prefs).checkstudyUID eq 1 ) then begin
		allsets = (*g).imgSets->get( /all, count=nsets )
		for i = 0, nsets-1 do begin
			if( obj_isa( allsets[i], 'mrImageSetDicom' )) then begin
				if( newset->studyUID() ne allsets[i]->studyUID() ) then begin
					res = mrdialog_select([i18n->get('lps_studyUIDdoesNotMatch1'),$
					                       i18n->get('lps_studyUIDdoesNotMatch2'),$
					                       i18n->get('lps_studyUIDdoesNotMatch3'),$
					                       i18n->get('lps_studyUIDdoesNotMatch4')],$
					                      [i18n->get('lps_keepOldData'), i18n->get('lps_keepNewData')],$
			    	    	              /question, group_leader=(*g).topid, title=i18n->get('lps_noMatchTitle') )
			    	; keep old data?
					if( res eq 0 ) then begin
						obj_destroy, newset
						return
					end
					; otherwise, destroy all existing image sets
					for j = 0, nsets-1 do begin
						(*g).imgSets->remove, allsets[j]
						obj_destroy, allsets[j]
					end
				end
				break ; only need to check one - they should (!) be identical
			end
		end
	end

		; destroy all data of class 'raw' as these are all derived from the dataset just loaded

	rawsets = mrGetImageSet( (*g).imgSets, class='raw', count=nrawsets )
	for i = 0, nrawsets-1 do begin
		(*g).imgSets->remove, rawsets[i]
		obj_destroy, rawsets[i]
	end
	(*g).imgSets->add, newset

		; after successful loading, remember the directory

	(*g).workingdir = path

		; if preferences -> auto_calculate_concentration_images is set, then do exactly that now
		; either for CT or for MR DSC acquired data

	if( (*prefs).autocalcctr eq 1 ) then begin
		modcalc = (*g).modules->get('mod_calc')
		case option of
			'DSC': modcalc->calcCtrImgsetFromT2star, /progressbar
			'CT' : modcalc->calcCtrImgsetFromCT, /progressbar
		end
	end

end

;+=----------------------------------------------------------------------------
; Name  : mod_loadperfseq__define
; Desc  : Private Procedure (constructor)
;=-----------------------------------------------------------------------------
pro mod_loadperfseq__define
	obj = { MOD_LOADPERFSEQ    ,$
			inherits LUPEMOD    $
		  }
end

